#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::Cookbooks::ExpandedCookbook
#
# The idea behind this class is that we can make use of the expanded
# cookbook dataset. This avoids any sanitization steps and thus
# should be quite fast.
#
# Usage example:
#
#   RBT::Action::Cookbooks::ExpandedCookbook.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/cookbooks/expanded_cookbook/expanded_cookbook.rb'
# RBT::Action::Cookbooks::ExpandedCookbook.new(ARGV)
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

module Cookbooks

class ExpandedCookbook < RBT::Action # === RBT::Action::Cookbooks::ExpandedCookbook

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true,
      &block
    )
    reset
    set_commandline_arguments(i)
    case run_already
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet
      run_already = false
    end
    # ======================================================================= #
    # === Handle blocks given to this class next
    # ======================================================================= #
    if block_given?
      _yielded = yield
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === program_path?
  # ========================================================================= #
  def program_path?
    @internal_hash[:dataset]['program_path']
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @internal_hash[:dataset]
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    # ======================================================================= #
    # === :dataset
    #
    # We load the dataset from an already existing .yml file next. Naturally
    # this will only work if the .yml file exists, and has the desired
    # dataset, as-is.
    # ======================================================================= #
    @internal_hash[:dataset] = load_dataset_from_this_expanded_cookbook(first_argument?)
  end

  # ========================================================================= #
  # === RBT::Action::Cookbooks::ExpandedCookbook[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

# =========================================================================== #
# === RBT.expanded_cookbook
# =========================================================================== #
def self.expanded_cookbook(
    i = ARGV,
    &block
  )
  return RBT::Action::Cookbooks::ExpandedCookbook.new(i, &block)
end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  expanded_cookbook = RBT::Action::Cookbooks::ExpandedCookbook.new(ARGV)
  if RBT.is_on_roebe?
    e
    pp expanded_cookbook.dataset? # Show the dataset here.
    e
  end
end # expandedcookbook htop