#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::Extract
# =========================================================================== #
# require 'rbt/actions/individual_actions/extract/extract.rb'
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class Extract < RBT::Action # === RBT::Action::Extract

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true,
      &block
    )
    reset
    case i
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet
      i = nil
      run_already = false
    end
    set_commandline_arguments(i)
    do_assume_that_all_commandline_arguments_are_programs_we_seek_to_extract
    # ======================================================================= #
    # === Handle blocks given to this class next
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :do_not_run_yet
      # ===================================================================== #
      when :do_not_run_yet
        run_already = false
      # ===================================================================== #
      # === :be_silent
      #
      # This entry point forces this class to be quiet - aka to output
      # significantly less to the user.
      # ===================================================================== #
      when :be_silent,
           :be_quiet
        @internal_hash[:be_silent_or_verbose] = :be_silent
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    try_to_require_the_extracter_gem
    # ======================================================================= #
    # === :extract_these_programs
    # ======================================================================= #
    @internal_hash[:extract_these_programs] = []
    # ======================================================================= #
    # === :be_silent_or_verbose
    # ======================================================================= #
    @internal_hash[:be_silent_or_verbose] = :be_verbose
  end

  # ========================================================================= #
  # === be_silent_or_verbose?
  # ========================================================================= #
  def be_silent_or_verbose?
    @internal_hash[:be_silent_or_verbose]
  end

  # ========================================================================= #
  # === set_extract_these_programs
  # ========================================================================= #
  def set_extract_these_programs(i)
    unless @internal_hash[:extract_these_programs].include? i
      @internal_hash[:extract_these_programs] << i
      @internal_hash[:extract_these_programs].flatten!
      @internal_hash[:extract_these_programs].compact
    end
  end; alias set_extract_this_program set_extract_these_programs # === set_extract_this_program

  # ========================================================================= #
  # === do_assume_that_all_commandline_arguments_are_programs_we_seek_to_extract
  # ========================================================================= #
  def do_assume_that_all_commandline_arguments_are_programs_we_seek_to_extract
    _ = commandline_arguments?
    if _ and !_.empty?
      set_extract_these_programs(_)
    end
  end

  # ========================================================================= #
  # === extract_these_programs?
  # ========================================================================= #
  def extract_these_programs?
    @internal_hash[:extract_these_programs]
  end

  # ========================================================================= #
  # === try_to_find_an_alternative_path_for_this_source_archive
  # ========================================================================= #
  def try_to_find_an_alternative_path_for_this_source_archive(entry)
    # =================================================================== #
    # Assume pseudo-symbol here, such as :htop
    # =================================================================== #
    entry = "#{src_dir?}#{entry.to_s.delete(':')}/"
    # =================================================================== #
    # We get the first entry anyway.
    # =================================================================== #
    possible_files = Dir["#{entry}*"]
    entry = possible_files.first unless possible_files.empty?
    return entry
  end

  # ========================================================================= #
  # === message_to_the_user_next_extracting_to_this_directory
  # ========================================================================= #
  def message_to_the_user_next_extracting_to_this_directory(
      entry, extract_to
    )
    opne "#{rev}Next extracting #{sfile(entry)}#{rev} "\
         "to #{sdir(extract_to)}#{rev}."
  end

  # ========================================================================= #
  # === path?
  # ========================================================================= #
  def path?
    @extract_to
  end

  # ========================================================================= #
  # === delegate_towards_class_extracter
  #
  # This method will delegate towards class Extracter.
  # ========================================================================= #
  def delegate_towards_class_extracter(
      what,
      extract_to = :default
    )
    case what
    # ======================================================================= #
    # === :default_programs
    # ======================================================================= #
    when :default_programs
      what = extract_these_programs?
    end
    flattened = [what].flatten.compact
    flattened.each {|this_entry|
      if this_entry.is_a?(String) and this_entry.start_with?(':')
        this_entry = try_to_find_an_alternative_path_for_this_source_archive(this_entry.delete(':'))
      elsif this_entry.is_a?(Symbol)
        this_entry = try_to_find_an_alternative_path_for_this_source_archive(this_entry.to_s)
      end
      case extract_to
      # ======================================================================= #
      # === :default
      # ======================================================================= #
      when :default
        extract_to = extract_to?(this_entry)
      end
      # ======================================================================= #
      # Finally, delegate towards Extracter:
      # ======================================================================= #
      Extracter.extract_what_to(this_entry, to: extract_to) { be_silent_or_verbose? }
    }
  end; alias invoke_class_extracter delegate_towards_class_extracter # === invoke_class_extracter

  # ========================================================================= #
  # === set_extract_to
  # ========================================================================= #
  def set_extract_to(i)
    @extract_to = i
  end

  # ========================================================================= #
  # === do_extract_all_programs_supplied_to_this_class
  # ========================================================================= #
  def do_extract_all_programs_supplied_to_this_class(
      i = extract_these_programs?
    )
    i.flatten.each {|entry|
      if entry.is_a?(String) and entry.start_with?(':')
        # =================================================================== #
        # Query RawCookbook for the assumed path next:
        # =================================================================== #
        raw_cookbook = action(:raw_cookbook, entry.to_s.delete(':'))
        program_path = raw_cookbook.program_path?
        if File.file?(program_path)
          @extract_to = extract_to?(entry).delete(':')
          message_to_the_user_next_extracting_to_this_directory(program_path, @extract_to)
          return delegate_towards_class_extracter(program_path, @extract_to)
        else
          entry = try_to_find_an_alternative_path_for_this_source_archive(entry)
        end
      end
      entry = entry.first if entry.is_a?(Array)
      if is_this_an_archive? entry
        entry = absolute_path(entry) # Work with the absolute path here.
        # =================================================================== #
        # The target is an archive, so we can extract it:
        # =================================================================== #
        @extract_to = extract_to?(entry)
        message_to_the_user_next_extracting_to_this_directory(entry, @extract_to)
        _result = delegate_towards_class_extracter(entry, @extract_to)
      else
        opne sfile(entry)+rev+' does not appear to be a registered archive type.'
      end
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    do_extract_all_programs_supplied_to_this_class
  end

end; end

# =========================================================================== #
# === RBT.handle_this_extract_related_user_input
#
# This method will either extract the given user input, such as
# "htop-2.2.tar.gz", or it may attempt to download the remote
# dataset associated with a given program at hand.
# =========================================================================== #
def self.handle_this_extract_related_user_input(
    i
  )
  i = [i].flatten.compact.each {|user_input|
    if File.exist? user_input
      RBT.extract(user_input)
    else
      RBT.download(user_input.to_sym)
      possible_matches = Dir[user_input.to_s+'*']
      this_directory = possible_matches.select {|line|
        File.file?(line) and is_an_archive?(line)
      }.first
      RBT.extract(this_directory)
      exit
    end
  }
end

# =========================================================================== #
# === RBT.extract
#
# This toplevel-method can be used to extract an archive, such as
# "foobar-1.2.3.tar.xz".
#
# The functionality depends on the gem called extracter, so users have
# to install that gem - for instance via "gem install extracter".
#
# The second argument to this method should be the target directory
# onto which the archive will be extracted to. So, for instance,
# if the argument is '/tmp/', then the archive will be extracted
# right into that directory.
#
# Usage examples:
#
#   RBT.extract(ARGV)
#   RBT.extract('/tmp/foobar-1.0.tar.xz','/Depot/j/')
#
# =========================================================================== #
def self.extract(
    optional_argument1,
    extract_to = :default, # Where to extract the archive at hand.
    &block
  )
  _ = RBT::Action::Extract.new(:do_not_run_yet, &block)
  _.set_extract_this_program(optional_argument1)
  _.invoke_class_extracter(:default_programs, extract_to)
  return _
end; self.instance_eval { alias extract_what_to extract } # === RBT.extract_what_to

end

if __FILE__ == $PROGRAM_NAME
  _extract = RBT::Action::Extract.new(ARGV)
end # actionextract htop*