#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::FindLikelyDirectory
#
# This class will find a most likely directory if the input match
# is not 100% accurate.
#
# Usage example:
#
#   RBT::Action::FindLikelyDirectory.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/find_likely_directory/find_likely_directory.rb'
# RBT::Action::FindLikelyDirectory.new(ARGV)
# RBT.find_likely_directory(ARGV)
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class FindLikelyDirectory < RBT::Action # === RBT::Action::FindLikelyDirectory

  # ========================================================================= #
  # === FindLikelyDirectory::SOURCE_DIRECTORY
  # ========================================================================= #
  SOURCE_DIRECTORY = ENV['MY_SRC'].to_s

  # ========================================================================= #
  # === FindLikelyDirectory::TEMP_DIRECTORY
  # ========================================================================= #
  TEMP_DIRECTORY   = ENV['MY_TEMP'].to_s

  # ========================================================================= #
  # === FindLikelyDirectory::DEFAULT_CHOICE
  # ========================================================================= #
  DEFAULT_CHOICE   = "#{TEMP_DIRECTORY}/binutils-2.*"

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = DEFAULT_CHOICE,
      run_already = true
    )
    set_find_similarities_for_this_string(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === string?
  # ========================================================================= #
  def string?
    @find_similarities_for_this_string
  end

  # ========================================================================= #
  # === set_find_similarities_for_this_string
  # ========================================================================= #
  def set_find_similarities_for_this_string(i = DEFAULT_CHOICE)
    if i.is_a? Array
      i = i.first
    end
    i = DEFAULT_CHOICE if i.nil?
    @find_similarities_for_this_string = i
  end

  # ========================================================================= #
  # === find_similarities
  #
  # This method does the searching part.
  # ========================================================================= #
  def find_similarities
    splitted = @find_similarities_for_this_string.split('-')
    _ = splitted.first # get all chars before the first - to use as glob. 
    obtain_matches(_)
  end

  # ========================================================================= #
  # === obtain_matches
  # ========================================================================= #
  def obtain_matches(path) # This sets @matches
    @matches = Dir["#{path}*"]
    @matches.select! {|entry| File.directory?(entry) } # We only care about directories.
  end

  # ========================================================================= #
  # === match
  # ========================================================================= #
  def match
    return @matches.first # right now will always return the first member.
  end

  # ========================================================================= #
  # === found_match?
  # ========================================================================= #
  def found_match? # return bool
    if @matches.empty?
      return false
    else
      return true
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    find_similarities
  end

  # ========================================================================= #
  # === RBT::Action::RawCookbook[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.find_likely_directory
# =========================================================================== #
def self.find_likely_directory(
    i            = ARGV,
    options_hash = {},
    &block
  )
  RBT::Action::FindLikelyDirectory.new(i, &block)
end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  _ = RBT.find_likely_directory(ARGV)
  e "\n\n"+'We tried to find similarities for: '+
    "\n\n"+'  '+_.string?+"\n\n"
end # fld