#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::ReportMateDesktopVersion
#
# This class will report the components of the mate desktop available
# on the given host computer.
#
# Usage examples:
#
#   RBT::Action::ReportMateDesktopVersion.new(ARGV)
#   RBT.report_mate_desktop_version
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/information/report_mate_desktop_version.rb'
# RBT.report_mate_desktop_version
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class ReportMateDesktopVersion < RBT::Action # === RBT::Action::ReportMateDesktopVersion

  # ========================================================================= #
  # === N_LJUST
  # ========================================================================= #
  N_LJUST = 22

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true,
      &block
    )
    reset
    if block_given?
      yielded = yield
      case yielded
      when :do_not_report,
           :do_not_report_anything
        @do_report = false
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @n_ljust
    # ======================================================================= #
    @n_ljust   = N_LJUST
    # ======================================================================= #
    # === :string_to_display
    # ======================================================================= #
    @internal_hash[:string_to_display] = ''.dup
    # ======================================================================= #
    # === @do_report
    # ======================================================================= #
    @do_report = true
  end

  # ========================================================================= #
  # === add                                                         (add tag)
  #
  # This method will simply append onto the corresponding variable, stored
  # in the primary Hash.
  # ========================================================================= #
  def add(
      i           = '',
      add_newline = true
    )
    case add_newline
    # === :no_newline
    when :no_newline
      add_newline = false
    end
    if add_newline
      i = i.dup if i.frozen?
      i << "\n"
    end
    @internal_hash[:string_to_display] << i
  end

  # ========================================================================= #
  # === string_to_display?
  # ========================================================================= #
  def string_to_display?
    @internal_hash[:string_to_display]
  end; alias string? string_to_display? # === string?
       alias result? string_to_display? # === result?

  # ========================================================================= #
  # === add_newline
  # ========================================================================= #
  def add_newline
    add
  end

  # ========================================================================= #
  # === left_padded
  # ========================================================================= #
  def left_padded(
      i = rev
    )
    "#{i}  " # This used to be "grey" rather than "rev", up until January 2024.
  end

  # ========================================================================= #
  # === report_caja_version
  # ========================================================================= #
  def report_caja_version
    add left_padded+'Caja: '.ljust(@n_ljust)+slateblue(cajav?)
  end

  # ========================================================================= #
  # === report_libcaja_extensions
  # ========================================================================= #
  def report_libcaja_extensions
    add left_padded+'Caja-extensions: '.ljust(@n_ljust)+slateblue(caja_extensions?)
  end

  # ========================================================================= #
  # === report_caja_dropbox_version
  # ========================================================================= #
  def report_caja_dropbox_version
    add left_padded+'Caja-dropbox: '.ljust(@n_ljust)+slateblue(cajadropboxv?)
  end

  # ========================================================================= #
  # === report_engrampa_version
  # ========================================================================= #
  def report_engrampa_version
    add left_padded+'Engrampa: '.ljust(@n_ljust)+slateblue(engrampav?)
  end

  # ========================================================================= #
  # === report_eom_version                                          (eom tag)
  # ========================================================================= #
  def report_eom_version
    add left_padded+'Eom: '.ljust(@n_ljust)+slateblue(eomv?)
  end

  # ========================================================================= #
  # === report_pluma_version
  # ========================================================================= #
  def report_pluma_version
    add left_padded+'Pluma: '.ljust(@n_ljust)+slateblue(plumav?)
  end

  # ========================================================================= #
  # === report_atril_version
  # ========================================================================= #
  def report_atril_version
    add left_padded+'Atril: '.ljust(@n_ljust)+slateblue(atrilv?)
  end

  # ========================================================================= #
  # === report_marco_version
  # ========================================================================= #
  def report_marco_version
    add left_padded+'Marco: '.ljust(@n_ljust)+slateblue(marcov?)
  end

  # ========================================================================= #
  # === report_mate_common_version
  # ========================================================================= #
  def report_mate_common_version
    add left_padded+'Mate-Common: '.ljust(@n_ljust)+slateblue(mate_common?)
  end

  # ========================================================================= #
  # === report_mate_keyboard_version
  # ========================================================================= #
  def report_mate_keyboard_version
    add left_padded+'Mate-Keyboard: '.ljust(@n_ljust)+slateblue(mate_keyboard?)
  end

  # ========================================================================= #
  # === report_mate_mixer_version
  # ========================================================================= #
  def report_mate_mixer_version
    add left_padded+'Mate-Mixer: '.ljust(@n_ljust)+slateblue(mate_mixer?)
  end

  # ========================================================================= #
  # === report_mate_media_version
  # ========================================================================= #
  def report_mate_media_version
    add left_padded+'Mate-Media: '.ljust(@n_ljust)+slateblue(mate_media?)
  end

  # ========================================================================= #
  # === report_mate_screensaver_version
  # ========================================================================= #
  def report_mate_screensaver_version
    add left_padded+'Mate-Screensaver: '.ljust(@n_ljust)+slateblue(mate_screensaver?)
  end

  # ========================================================================= #
  # === report_mate_weather_version
  # ========================================================================= #
  def report_mate_weather_version
    add left_padded+'Mate-Weather: '.ljust(@n_ljust)+slateblue(mate_weather?)
  end

  # ========================================================================= #
  # === report_mate_power_manager_version
  # ========================================================================= #
  def report_mate_power_manager_version
    add left_padded+'Mate-Power-Manager: '.ljust(@n_ljust)+slateblue(mate_power_manager?)
  end

  # ========================================================================= #
  # === report_mate_applets_version
  # ========================================================================= #
  def report_mate_applets_version
    add left_padded+'Mate-Applets: '.ljust(@n_ljust)+slateblue(mate_applets?)
  end

  # ========================================================================= #
  # === report_mate_control_center_version
  # ========================================================================= #
  def report_mate_control_center_version
    add left_padded+'Mate-Control-Center: '.ljust(@n_ljust)+slateblue(mate_control_center?)
  end

  # ========================================================================= #
  # === report_mate_panel_version
  # ========================================================================= #
  def report_mate_panel_version
    add left_padded+'Mate-Panel: '.ljust(@n_ljust)+slateblue(mate_panel?)
  end

  # ========================================================================= #
  # === return_version_of_mate_applets
  # ========================================================================= #
  def return_version_of_mate_applets
    version = nil
    _ = '/usr/share/mate-panel/applets/org.mate.applets.DriveMountApplet.mate-panel-applet'
    if File.exist? _
      version = File.read(_).scan(
        /X-MATE-Bugzilla-Version=(\d{1,2}\.?\d{1,2}\.?\d{1,2})/
      ).flatten.first.to_s
    end
    version
  end

  # ========================================================================= #
  # === run_safe_system_command
  #
  # The idea here is to append 2>&1, via ERROR_LINE.
  # ========================================================================= #
  def run_safe_system_command(
      i = 'mate-power-manager --version'
    )
    `#{i} #{ERROR_LINE}`.to_s.sub(/Version /,'')
  end

  # ========================================================================= #
  # === report_mate_desktop_version
  # ========================================================================= #
  def report_mate_desktop_version
    add left_padded+'Mate-Desktop: '.ljust(@n_ljust)+slateblue(mate_desktopv?)
  end

  # ========================================================================= #
  # === report_mate_utils_version
  # ========================================================================= #
  def report_mate_utils_version
    add left_padded+'Mate-Utils: '.ljust(@n_ljust)+slateblue(mate_utils?)
  end

  # ========================================================================= #
  # === string_to_display?
  # ========================================================================= #
  def string_to_display?
    @internal_hash[:string_to_display]
  end

  # ========================================================================= #
  # === mate_panel?
  # ========================================================================= #
  def mate_panel?
    return_sanitized_sys_command(:mate_panel)
  end; alias mate_panel_version? mate_panel? # === mate_panel_version?

  # ========================================================================= #
  # === mate_control_center?
  #
  # This method will return the version of the mate-control-center.
  # ========================================================================= #
  def mate_control_center?
    return_sanitized_sys_command(:mate_control_center)
  end; alias mate_control_center_version? mate_control_center? # === mate_control_center_version?
       alias mate_controlcenter_version?  mate_control_center? # === mate_controlcenter_version?

  # ========================================================================= #
  # === return_mate_media_version
  #
  # This is a bit volatile since we depend on "mate-volume-control"
  # here. It can lead to problems such as:
  #
  #   "mate-volume-control: symbol lookup error: /usr/lib/../lib/libpangoft2-1.0.so.0: undefined symbol: FcWeightFromOpenTypeDouble\n"
  #
  # That is why we have to be a bit more carefeul when determining
  # the version in place.
  # ========================================================================= #
  def return_mate_media_version
    result = `mate-volume-control --version #{ERROR_LINE}`.to_s
    if result.include? 'symbol lookup error'
      result = result.split('symbol lookup error').last
    elsif result.include? ' '
      result = result.strip.split(' ').last
    end
    result
  end; alias mate_media_version? return_mate_media_version # === mate_media_version?

  # ========================================================================= #
  # === mate_applets?
  # ========================================================================= #
  def mate_applets?
    return_sanitized_sys_command(:mate_applets)
  end; alias mate_applets_version? mate_applets? # === mate_applets_version?

  # ========================================================================= #
  # === mate_media?
  # ========================================================================= #
  def mate_media?
    return_sanitized_sys_command(:mate_media)
  end

  # ========================================================================= #
  # === mate_screensaver?
  # ========================================================================= #
  def mate_screensaver?
    return_sanitized_sys_command(:mate_screensaver)
  end

  # ========================================================================= #
  # === mate_utils?
  # ========================================================================= #
  def mate_utils?
    return_sanitized_sys_command(:mate_utils)
  end; alias mate_utils_version? mate_utils? # === mate_utils_version?

  # ========================================================================= #
  # === mate_common?
  # ========================================================================= #
  def mate_common?
    return_sanitized_sys_command(:mate_common)
  end; alias mate_common_version? mate_common? # === mate_common_version?

  # ========================================================================= #
  # === mate_desktopv?
  # ========================================================================= #
  def mate_desktopv?
    return_sanitized_sys_command(:mate_desktop)
  end; alias mate_desktop_version?          mate_desktopv? # === mate_desktop_version?
       alias return_version_of_mate_desktop mate_desktopv? # === return_version_of_mate_desktop

  # ========================================================================= #
  # === mate_keyboard?
  #
  # Return the version of libmatekbd.
  # ========================================================================= #
  def mate_keyboard?
    return_sanitized_sys_command(:mate_keyboard)
  end; alias mate_keyboard_version? mate_keyboard? # === mate_keyboard_version?
       alias libmatekbd_version?    mate_keyboard? # === libmatekbd_version?

  # ========================================================================= #
  # === mate_mixer?
  # ========================================================================= #
  def mate_mixer?
    return_sanitized_sys_command(:mate_mixer)
  end; alias mate_mixer_version?   mate_mixer? # === mate_mixer_version?
       alias libmatemixer_version? mate_mixer? # === libmatemixer_version?

  # ========================================================================= #
  # === mate_weather?
  # ========================================================================= #
  def mate_weather?
    return_sanitized_sys_command(:mate_weather)
  end; alias libmateweather_version? mate_weather? # === libmateweather_version?

  # ========================================================================= #
  # === mate_power_manager?
  # ========================================================================= #
  def mate_power_manager?
    return_sanitized_sys_command(:mate_power_manager)
  end

  # ========================================================================= #
  # === marcov?
  # ========================================================================= #
  def marcov? 
    return_sanitized_sys_command(:marco)
  end; alias mate_marco_version? marcov? # === mate_marco_version?
       alias marco_version?      marcov? # === marco_version?

  # ========================================================================= #
  # === atrilv?
  # ========================================================================= #
  def atrilv?
    return_sanitized_sys_command(:atril)
  end; alias mate_atril_version? atrilv? # === mate_atril_version?
       alias atril_version?      atrilv? # === atril_version?

  # ========================================================================= #
  # === eomv?
  # ========================================================================= #
  def eomv?
    return_sanitized_sys_command(:eom)
  end; alias mate_eom_version? eomv? # === mate_eom_version?
       alias eom_version?      eomv? # === eom_version?

  # ========================================================================= #
  # === plumav?
  # ========================================================================= #
  def plumav?
    return_sanitized_sys_command(:pluma)
  end; alias mate_pluma_version? plumav? # === mate_pluma_version?
       alias pluma_version?      plumav? # === pluma_version?

  # ========================================================================= #
  # === cajav?
  # ========================================================================= #
  def cajav? # Return the version of caja
    return_sanitized_sys_command(:caja)
  end; alias mate_caja_version? cajav? # === mate_caja_version?
       alias caja_version?      cajav? # === caja_version?

  # ========================================================================= #
  # === caja_extensions?
  #
  # Report the version of libcaja-extensions in use (if installed).
  # ========================================================================= #
  def caja_extensions?
    return_sanitized_sys_command(:caja_extensions)
  end

  # ========================================================================= #
  # === cajadropboxv?
  # ========================================================================= #
  def cajadropboxv?
    return_sanitized_sys_command(:caja_dropbox)
  end; alias mate_cajadropbox_version? cajadropboxv? # === mate_cajadropbox_version?
       alias caja_dropbox_version?     cajadropboxv? # === caja_dropbox_version?

  # ========================================================================= #
  # === array?
  # ========================================================================= #
  def array?
    # ======================================================================= #
    # The Array is currently hardcoded here. This may have to be changed
    # at a later time - but for the time being we will stick to that
    # approach.
    # ======================================================================= #
    array = %i(
      mate_desktop
      mate_utils
      mate_common
      libmatekbd
      mate_media
      libmatemixer
      libmateweather
      mate_applets
      mate_control_center
      marco
      atril
      eom
      pluma
      caja
      caja_dropbox
    )
    # ======================================================================= #
    # Now that we have the array, we will self-send here, by simply
    # appending a '?' to the method call. For this to work, evidently
    # such methods must exist. Sine I am unsure of conflicts, I have
    # also appended "_version?" there instead.
    # ======================================================================= #
    array.map! {|name_of_the_program|
      assumed_version_of_this_program = send("#{name_of_the_program}_version?")
      # ===================================================================== #
      # Next we will modify the above a bit, for empty strings and
      # for "command not found" strings.
      # ===================================================================== #
      if assumed_version_of_this_program.empty? or
         assumed_version_of_this_program.include?('command not found')
        assumed_version_of_this_program = nil
      end
      if assumed_version_of_this_program
        [
          name_of_the_program, assumed_version_of_this_program
        ]
      else
        nil
      end
    }
    array.compact!
    array
  end

  # ========================================================================= #
  # === engrampav?
  # ========================================================================= #
  def engrampav?
    return_sanitized_sys_command(:engrampa)
  end; alias mate_engrampa_version? engrampav? # === mate_engrampa_version?

  # ========================================================================= #
  # === return_pkgconfig_result_for
  #
  # Note that this method will NOT modify the input given, so you have
  # to ensure that the input is correct BEFORE calling this method.
  # ========================================================================= #
  def return_pkgconfig_result_for(
      i = :libmatekbd
    )
    result = `pkg-config --modversion #{i} #{ERROR_LINE}`.strip
    if result.include?('No package ') and result.include?(' found')
      result = ''
    end
    return result
  end

  # ========================================================================= #
  # === return_sanitized_sys_command
  #
  # This method will return the proper version of an installed
  # mate-desktop component.
  # ========================================================================= #
  def return_sanitized_sys_command(
      i = :caja
    )
    case i # case tag
    when :mate_common
      i = :mate_doc_common
    when :mate_utils
      i = :mate_screenshot
    when :mate_desktop
      i = :mate_about
    end
    # ======================================================================= #
    # Next, some packages such as libmatekbd, only come with a .pc file
    # rather than a commandline way to query its version, which we will
    # determine first.
    # ======================================================================= #
    case i
    # ======================================================================= #
    # === :mate_screensaver
    # ======================================================================= #
    when :mate_screensaver
      result = return_pkgconfig_result_for('mate-screensaver')
    # ======================================================================= #
    # === :mate_keyboard
    # ======================================================================= #
    when :mate_keyboard
      result = return_pkgconfig_result_for(:libmatekbd)
    # ======================================================================= #
    # === :mate_panel
    # ======================================================================= #
    when :mate_panel
      result = return_pkgconfig_result_for('libmatepanelapplet-4.0')
    # ======================================================================= #
    # === :caja_extensions
    # ======================================================================= #
    when :caja_extensions
      result = return_pkgconfig_result_for('libcaja-extension')
    # ======================================================================= #
    # === :mate_media
    # ======================================================================= #
    when :mate_media
      result = return_mate_media_version
    # ======================================================================= #
    # === :mate_power_manager
    # ======================================================================= #
    when :mate_power_manager
      result = run_safe_system_command('mate-power-manager --version').sub(/Version /,'')
    # ======================================================================= #
    # === :mate_mixer
    # ======================================================================= #
    when :mate_mixer
      result = return_pkgconfig_result_for(:libmatemixer)
    # ======================================================================= #
    # === :mate_weather
    # ======================================================================= #
    when :mate_weather
      result = return_pkgconfig_result_for(:mateweather)
    # ======================================================================= #
    # === :mate_applets
    # ======================================================================= #
    when :mate_applets
      result = return_version_of_mate_applets
    # ======================================================================= #
    # === :mate_control_center
    # ======================================================================= #
    when :mate_control_center
      result = return_pkgconfig_result_for('mate-default-applications')
    else
      i = i.to_s.tr('_','-')
      result = `#{i} --version #{ERROR_LINE}`.split(N).first # system tag
    end
    result.to_s.
           sub(/MATE /,'').
           sub(/caja /,'').
           sub(/marco /,'').
           sub(/mate-screenshot /,'').
           sub(/mate-doc-common \(mate-common\) /,''). 
           sub(/Desktop Environment /,'').
           sub(/Document Viewer /,'').
           sub(/Eye of Image Viewer /,'').
           sub(/pluma - Version /,'').
           sub(/Engrampa Archive Manager /,'').
           strip
  end

  # ========================================================================= #
  # === report                                                   (report tag)
  # ========================================================================= #
  def report(
      i         = string_to_display?, # This is the full String to be displayed.
      do_report = @do_report
    )
    if do_report
      e lightgreen(i)
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    add grey, :no_newline
    add "#{rev}Reporting the installed versions of the "\
        "#{royalblue('mate-desktop components')}"\
        "#{rev}:"
    add_newline
    report_mate_desktop_version        # === mate-desktop
    report_mate_utils_version          # === mate-utils
    report_mate_common_version         # === mate-common
    report_mate_keyboard_version       # === libmatekbd
    report_mate_media_version          # === mate-media
    report_mate_mixer_version          # === libmatemixer
    report_mate_applets_version        # === mate-applets
    report_mate_weather_version        # === libmateweather
    report_mate_control_center_version # === mate-control-center
    report_mate_panel_version          # === mate-panel
    report_mate_power_manager_version  # === mate-power-manager
    report_marco_version               # === marco
    report_mate_screensaver_version    # === mate-screensaver-version
    report_atril_version               # === atril
    report_eom_version                 # === eom
    report_pluma_version               # === pluma
    # ======================================================================= #
    # Engrampa works via the --version switch, which was different in ~2019.
    # So we can enable it finally.
    # ======================================================================= #
    report_engrampa_version            # === engrampa
    report_caja_version                # === caja
    report_libcaja_extensions          # === libcaja_extensions
    # report_caja_dropbox_version      # === caja-dropbox
    add_newline
    report
  end

  # ========================================================================= #
  # === RBT::Action::ReportMateDesktopVersion[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.return_mate_desktop_version_array                       (array tag)
#
# This variant will return a nested Array with the proper versions
# in use.
#
# Invocation example:
#
#   puts RBT.return_mate_desktop_version_array
#
# =========================================================================== #
def self.return_mate_desktop_version_array
  RBT::Action::ReportMateDesktopVersion.new { :do_not_report_anything }.array?
end

# =========================================================================== #
# === RBT.return_mate_desktop_version_string
#
# This method will return the String that contains the information which
# mate-desktop components are available. That way we can then embed
# this String in other programs.
#
# Usage example:
#
#   x = RBT.return_mate_desktop_version_string
#
# =========================================================================== #
def self.return_mate_desktop_version_string
  RBT::Action::ReportMateDesktopVersion.new { :do_not_report_anything }.string?
end

# =========================================================================== #
# === RBT.report_mate_desktop_version
# =========================================================================== #
def self.report_mate_desktop_version
  RBT::Action::ReportMateDesktopVersion.new
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.report_mate_desktop_version
end # report_mate_desktop_version