#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::ShowVersionsOfThesePrograms
#
# Usage examples:
#
#   RBT::Action::ShowVersionsOfThesePrograms.new(ARGV)
#   RBT.show_versions_of_these_programs
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/information/show_versions_of_these_programs.rb'
# RBT.show_versions_of_these_programs(ARGV)
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class ShowVersionsOfThesePrograms < RBT::Action # === RBT::Action::ShowVersionsOfThesePrograms

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_commandline_arguments(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === do_work_on_the_programs
  # ========================================================================= #
  def do_work_on_the_programs(
      i = commandline_arguments?
    )
    e
    i.each {|this_program|
      if RBT.respond_to? :swift_return_program_version
        version = RBT.swift_return_program_version(this_program)
      else
        version = action(:SanitizeCookbook, this_program) { :fast }.version?
      end
      this_program = (
        "#{File.basename(this_program).sub(/\.yml$/,'')}:"
      ).ljust(30)
      e "  #{steelblue(this_program)} "+
        mediumpurple(version.to_s)
    }
    e
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      when /^-?-?all$/i
        commandline_arguments? << available_programs?
        commandline_arguments?.flatten!
      end
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    if commandline_arguments?.empty?
      opne "#{rev}Please provide at the least one argument to this class."
    else
      do_work_on_the_programs
    end
  end

  # ========================================================================= #
  # === RBT::Action::ShowVersionsOfThesePrograms[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.show_versions_of_these_programs
# =========================================================================== #
def self.show_versions_of_these_programs(i = ARGV)
  RBT::Action::ShowVersionsOfThesePrograms.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.show_versions_of_these_programs(ARGV)
end # showversionsoftheseprograms htop ruby php