#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::Statistics::ShowCompileTimeStatistics
#
# Usage example:
#
#   RBT::Action::Statistics::ShowCompileTimeStatistics.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/statistics/show_compile_time_statistics.rb'
# RBT::Action::Statistics::ShowCompileTimeStatistics.new(ARGV)
# RBT.show_compile_time_statistics
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

module Statistics

class ShowCompileTimeStatistics < RBT::Action # === RBT::Action::Statistics::ShowCompileTimeStatistics

  require 'rbt/actions/individual_actions/statistics/return_compile_time_statistics.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true,
      &block
    )
    reset
    set_commandline_arguments(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === gray_header
  # ========================================================================= #
  def gray_header(
      n_times = 90
    )
    e gray('-' * n_times)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    _ = RBT.return_compile_time_statistics
    unless _.does_the_main_file_exist?
      opne "#{rev}No file exists at `#{sfile(_.main_file?)}#{rev}`."
      opne 'We thus can not show any statistics.'
      return
    end
    if _.empty?
      opne "#{rev}The dataset is empty. Have there any programs been "\
           "compiled yet, through #{teal('rbt')}#{rev}?"
    else
      opnn { :no_trailing }
      e; e
      e "#{rev}Showing the compile-time statistics for the following "\
        "#{sfancy(_.keys.size.to_s)} #{rev}programs."
      e
      # ===================================================================== #
      # Show a little header explaining what this is all about.
      # ===================================================================== #
      gray_header
      # ===================================================================== #
      # Next, properly name the header-entries
      # ===================================================================== #
      header = _.return_the_header.dup
      header.sub!(/(Program name)/,          sfancy('\\1')+rev)
      header.sub!(/(Compile time \[sec\] )/, simp('\\1')+rev)
      header.sub!(/(Compile time \[min\] )/, orange('\\1')+rev)
      header.sub!(/(Archive file size)/,     mediumslateblue('\\1')+rev)
      header.sub!(/(#)/,                     mediumorchid('#')+rev)
      e header
      gray_header
      e _.body?
    end        
  end

  # ========================================================================= #
  # === RBT::Action::Statistics::ShowCompileTimeStatistics[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

# =========================================================================== #
# === RBT.show_compile_time_statistics
# =========================================================================== #
def self.show_compile_time_statistics
  RBT::Action::Statistics::ShowCompileTimeStatistics.new
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.show_compile_time_statistics
end # showcompiletimestatistics
