#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::CheckForGnomeUpdates
#
# This class will check for new updates on the gnome ftp server.
#
# Every new version it finds will be downloaded.
#
# You can trigger this from the commandline by issuing:
#
#   cookbooks --check-for-gnome-updates
#
# =========================================================================== #
# require 'NEW_check_for_gnome_updates.rb'
# =========================================================================== #
# TODO: STILL HAS TO BE FINISHED.
# =========================================================================== #
require 'rbt/check_for_updates/base.rb'

module RBT

module Cookbooks

class NewCheckForGnomeUpdates < CheckForRemoteWebpages # === Cookbooks::NewCheckForGnomeUpdates.new

  # ========================================================================= #
  # === TRAILING_END_OF_THE_REMOTE_WEBPAGE
  # ========================================================================= #
  TRAILING_END_OF_THE_REMOTE_WEBPAGE = '?C=M;O=D'

  # ========================================================================= #
  # === ONE_DAY
  # ========================================================================= #
  ONE_DAY      = 24 * 60 * 60 # 24 hours: one hour has 3600 seconds.

  # ========================================================================= #
  # === USE_ONLY_STABLE_GNOME_VERSIONS
  #
  # If the following constant is set to true, we will only use stable
  # gnome versions for the purpose of this class here. So for example,
  # any program with version names such "2.27.1" will be ignored.
  # The user will be notified about this on startup.
  # ========================================================================= #
  USE_ONLY_STABLE_GNOME_VERSIONS = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_commandline_arguments(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @use_only_stable_gnome_versions
    # ======================================================================= #
    @use_only_stable_gnome_versions = USE_ONLY_STABLE_GNOME_VERSIONS
    @array_updated_these_entries = [] # <- Which programs were updated is kept in this Array.
  end

  # ========================================================================= #
  # === also_check_for_unstable_gnome_versions
  #
  # By default, this method will be silent. If you need to send a message
  # to the user, you have to do so on your own.
  # ========================================================================= #
  def also_check_for_unstable_gnome_versions
    @use_only_stable_gnome_versions = false
  end

  # ========================================================================= #
  # === consider_notifying_the_user_about_using_only_stable_gnome_versions
  # ========================================================================= #
  def consider_notifying_the_user_about_using_only_stable_gnome_versions
    if @use_only_stable_gnome_versions
      opne 'Only stable versions will be considered for this class, as the'
      opne 'variable '+orange('@use_only_stable_gnome_versions')+
           ' has been set to true.'
      opne 'If this is unwanted, pass in the option '+
           slateblue('--unstable')+' to this class.'
    end
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === new_gnome --unstable
      # ===================================================================== #
      when /^-?-?unstable$/
        opne 'Also checking for unstable versions of gnome-releases.'
        also_check_for_unstable_gnome_versions
      end
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    consider_notifying_the_user_about_using_only_stable_gnome_versions
  end

  # ========================================================================= #
  # === RBT::Cookbooks::NewCheckForGnomeUpdates[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::NewCheckForGnomeUpdates.new(ARGV)
end # NEW_check_for_gnome_updates