#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file will check for faulty current symlinks in the app-dir hierarchy.
# =========================================================================== #
# require 'rbt/toplevel_methods/check_for_faulty_current_symlinks.rb'
# =========================================================================== #
module RBT

  require 'rbt/toplevel_methods/e_and_esystem.rb'
  require 'rbt/toplevel_methods/opn.rb'
  require 'rbt/requires/require_colours.rb'
  require 'rbt/toplevel_methods/files_and_directories_related_methods.rb'

  # ========================================================================= #
  # === RBT.remove_symlink
  # ========================================================================= #
  def self.remove_symlink(i)
    if File.symlink? i
      File.delete(i)
    end
  end

  # ========================================================================= #
  # === RBT.check_for_faulty_current_symlinks
  #
  # This method is mostly an ad-hoc method. It will try to remove entries
  # such as /home/Programs/Libsndfile/Current1.0.31 specifically.
  # ========================================================================= #
  def self.check_for_faulty_current_symlinks(
      in_this_directory
    )
    if in_this_directory.is_a? Array
      in_this_directory = in_this_directory.first
    end
    if in_this_directory.nil?
      in_this_directory = ::RBT.programs_dir?
    end
    in_this_directory = in_this_directory.dup
    in_this_directory << '/' unless in_this_directory.end_with? '/'
    all_current_symlinks = Dir[in_this_directory+'**/**'].select {|entry|
      File.symlink?(entry) and entry.include?('/Current')
    }
    # ======================================================================= #
    # Next, distinguish between:
    #
    #   "/home/Programs/Libsndfile/Current"
    #
    # and:
    #
    #   "/home/Programs/Libsndfile/Current1.0.31"
    #
    # ======================================================================= #
    all_faulty_current_symlinks = all_current_symlinks.select {|entry|
      last_char = entry[-1, 1]
      (last_char =~ /\d+/)
    }
    if all_faulty_current_symlinks.empty?
      opn(namespace: 'RBT'); e 'No faulty symlinks were found.'
      opn(namespace: 'RBT'); e 'The directory that was checked was `'+
                               ::Colours.sdir(in_this_directory)+
                               rev+'`.'
    else
      opne 'Faulty symlinks were found. These will be removed next.'
      all_faulty_current_symlinks.each {|entry|
        opne 'Next removing the faulty symlink at '+entry+'.'
        remove_symlink(entry)
      }
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.check_for_faulty_current_symlinks(ARGV)  
end # checkforfaultycurrentsymlinks