#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/colours/rev_simp_sdir_sfancy_sfile_and_swarn.rb'
# =========================================================================== #
module RBT

  require 'rbt/requires/require_colours.rb'
  require 'rbt/colours/use_colours.rb'

  # ========================================================================= #
  # === COLOURS
  # ========================================================================= #
  COLOURS = ::Colours

  # ========================================================================= #
  # === HASH_DESIGNATED_COLOURS
  # ========================================================================= #
  HASH_DESIGNATED_COLOURS = {} 
  _ = RBT.file_colours
  if File.exist? _
    HASH_DESIGNATED_COLOURS.update(YAML.load_file(_))
  end

  # ========================================================================= #
  # === RBT.user_designated_colour
  #
  # Note that "error" and "information" is currently (January 2023) not
  # handled here.
  # ========================================================================= #
  def self.user_designated_colour(i = 'information')
    case i
    # ======================================================================= #
    # === :sfancy
    # ======================================================================= #
    when :sfancy
      i = 'fancy'
    # ======================================================================= #
    # === :sdir
    # ======================================================================= #
    when :sdir
      i = 'directory'
    # ======================================================================= #
    # === :sfile
    # ======================================================================= #
    when :sfile
      i = 'file'
    # ======================================================================= #
    # === :swarn
    # ======================================================================= #
    when :swarn
      i = 'warn'
    # ======================================================================= #
    # === :simp
    # ======================================================================= #
    when :simp
      i = 'important'
    end
    return HASH_DESIGNATED_COLOURS[i]
  end

  # ========================================================================= #
  # === RBT.simp
  # ========================================================================= #
  def self.simp(
      i           = '',
      use_colours = RBT.use_colours?
    )
    if use_colours?
      if HASH_DESIGNATED_COLOURS.empty?
        return COLOURS.simp(i)
      else
        return COLOURS::HtmlColoursMethods.send(user_designated_colour(:simp), i)
      end
    end
    return i
  end

  # ========================================================================= #
  # === RBT.swarn
  # ========================================================================= #
  def self.swarn(i = '')
    if use_colours?
      if HASH_DESIGNATED_COLOURS.empty?
        return COLOURS.swarn(i)
      else
        return COLOURS::HtmlColoursMethods.send(user_designated_colour(:swarn), i)
      end
    end
    return i
  end

  # ========================================================================= #
  # === swarn
  # ========================================================================= #
  def swarn(i = '')
    RBT.swarn(i)
  end

  # ========================================================================= #
  # === RBT.sdir
  # ========================================================================= #
  def self.sdir(i = '')
    if use_colours?
      if HASH_DESIGNATED_COLOURS.empty?
        return COLOURS.sdir(i)
      else
        return COLOURS::HtmlColoursMethods.send(user_designated_colour(:sdir), i)
      end
    end
    return i
  end

  # ========================================================================= #
  # === RBT.sfile
  # ========================================================================= #
  def self.sfile(i = '')
    if use_colours?
      if HASH_DESIGNATED_COLOURS.empty?
        return COLOURS.sfile(i)
      else
        return COLOURS::HtmlColoursMethods.send(user_designated_colour(:sfile), i)
      end
    end
    return i
  end

  # ========================================================================= #
  # === sfile
  # ========================================================================= #
  def sfile(i = '')
    RBT.sfile(i)
  end

  # ========================================================================= #
  # === RBT.ewarn
  # ========================================================================= #
  def self.ewarn(i = '')
    e ::RBT.swarn(i)
  end

  # ========================================================================= #
  # === RBT.sfancy
  # ========================================================================= #
  def self.sfancy(
      i = '', use_colours = use_colours?
    )
    if use_colours
      if HASH_DESIGNATED_COLOURS.empty?
        return COLOURS.sfancy(i)
      else
        return COLOURS::HtmlColoursMethods.send(user_designated_colour(:sfancy), i)
      end
    end
    return i
  end

  # ========================================================================= #
  # === RBT.efancy
  # ========================================================================= #
  def self.efancy(i = '')
    e sfancy(i)
  end

  # ========================================================================= #
  # === sfancy
  # ========================================================================= #
  def sfancy(i = '')
    RBT.sfancy(i)
  end

  # ========================================================================= #
  # === RBT.rev
  # ========================================================================= #
  def self.rev(
      use_colours = use_colours?
    )
    if use_colours
      return COLOURS.rev
    end
    return ''
  end

  # ========================================================================= #
  # === RBT.eimp
  # ========================================================================= #
  def self.eimp(
      i           = '',
      use_colours = RBT.use_colours?
    )
    e RBT.simp(i, use_colours)
  end

  # ========================================================================= #
  # === simp
  # ========================================================================= #
  def simp(i = '')
    RBT.simp(i)
  end

  # ========================================================================= #
  # === sdir
  # ========================================================================= #
  def sdir(i = '')
    RBT.sdir(i)
  end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  e RBT.swarn('Testing.')
end