#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/colours/use_colours.rb'
# =========================================================================== #
module RBT

  require 'rbt/toplevel_methods/e_and_esystem.rb'
  require 'rbt/project/project.rb'
  require 'rbt/requires/require_colours.rb'

  # ========================================================================= #
  # === @use_colours
  #
  # Define a toplevel "use colours" variable. By default we will use
  # colours.
  # ========================================================================= #
  @use_colours = true

  # ========================================================================= #
  # === RBT.use_colours?
  #
  # Query method in order to determine whether the RBT project will
  # make use of colours or whether it will not.
  # ========================================================================= #
  def self.use_colours?
    @use_colours
  end

  # ========================================================================= #
  # === RBT.file_colours
  #
  # This method will point to the .yml file that users can modify if
  # they want to use different colours.
  # ========================================================================= #
  def self.file_colours
    "#{RBT.project_yaml_directory?}colours/colours.yml"
  end

  # ========================================================================= #
  # === RBT.disable_colours
  #
  # This method can be used to disable the use of colours for the whole
  # RBT project.
  # ========================================================================= #
  def self.disable_colours
    @use_colours = false
  end; self.instance_eval { alias do_not_use_colours disable_colours } # === RBT.do_not_use_colours

end