#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CompileBase
#
# This class has been created so that we can use a unified subclass
# for classes such as:
#
#   - class RBT::Install
#   - class RBT::PostinstallationStep
#   - class RBT::Make
#
# Of course more classes can make use of CompileBase, so it really is
# a specialized variant that focuses on compile-related aspects.
#
# The idea was, then, to enable unified functionality for these
# classes, in particular for class RBT::PostinstallationStep,
# as I wanted to be able to tap into that functionality from
# the commandline as well.
#
# In February 2024 this was changed into a module, to allow
# mixing it into class Action.
#
# Usage example:
#
#   RBT::CompileBase.new(ARGV)
#
# =========================================================================== #
# require 'rbt/compile_base/compile_base.rb'
# < RBT::CompileBase
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class CompileBase < RBT::Base # === RBT::CompileBase

  require 'rbt/compile_base/compile_base_module.rb'
  include RBT::CompileBaseModule

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    reset_the_module if respond_to?(:reset_the_module)
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
  end

end; end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  compile_base = RBT::CompileBase.new(ARGV)
  e 'The editor in use is: '+compile_base.editor_to_use?.to_s
end # compile_base