#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/constants/constants.rb'
# RBT::FILE_SETUP_PY
# =========================================================================== #
module RBT

  require 'rbt/project/project.rb'
  require 'rbt/constants/configuration_directory.rb'
  require 'rbt/constants/namespace.rb'
  require 'rbt/requires/require_raw_yaml.rb'

  require 'rbt/toplevel_methods/home_directory.rb'

  project_yaml_directory = RBT.project_yaml_directory?

  # ========================================================================= #
  # === N
  # ========================================================================= #
  N = "\n"

  # ========================================================================= #
  # === TABULATOR
  # ========================================================================= #
  TABULATOR = "\t"

  # ========================================================================= #
  # === COMMAND_NOT_FOUND
  # ========================================================================= #
  COMMAND_NOT_FOUND = 'command not found'

  # ========================================================================= #
  # === RBT::HOME_FOR_X
  #
  # This constant is mostly used on my home setup, so it will be fairly
  # useless for other people.
  # ========================================================================= #
  HOME_FOR_X = '/home/x/'
    HOME_DIRECTORY_FOR_USER_X = HOME_FOR_X # === HOME_DIRECTORY_FOR_USER_X

  # ========================================================================= #
  # === DEPOT_DIR
  #
  # Hardcoded path to the depot-directory.
  # ========================================================================= #
  DEPOT_DIR = '/Depot/'

  # ========================================================================= #
  # === USR_INCLUDE_DIR
  #
  # This can be standalone because it will be ALWAYS absolute,
  # pointing towards "/usr/include/".
  # ========================================================================= #
  USR_INCLUDE_DIR = '/usr/include/'

  # ========================================================================= #
  # === FILE_SETUP_PY
  #
  # This constant can be useful if you have to install a program that
  # depends on the file 'setup.py'.
  # ========================================================================= #
  FILE_SETUP_PY = 'setup.py'

  # ========================================================================= #
  # === FILE_SCONS
  # ========================================================================= #
  FILE_SCONS = 'SConstruct'

  # ========================================================================= #
  # === FILE_WAF
  # ========================================================================= #
  FILE_WAF = 'waf'

  # ========================================================================= #
  # === FILE_CMAKE_LISTS
  # ========================================================================= #
  FILE_CMAKE_LISTS = 'CMakeLists.txt'

  # ========================================================================= #
  # === FILE_MESON_BUILD
  # ========================================================================= #
  FILE_MESON_BUILD = 'meson.build'

  # ========================================================================= #
  # === RBT.file_meson_build
  # ========================================================================= #
  def self.file_meson_build
    FILE_MESON_BUILD
  end

  # ========================================================================= #
  # === TAR_XZ
  #
  # Several archive types are kept here.
  # ========================================================================= #
  TAR_XZ  = '.tar.xz'
    TARXZ = TAR_XZ # === TARXZ

  # ========================================================================= #
  # === TAR_BZ2
  # ========================================================================= #
  TAR_BZ2 = '.tar.bz2'
    TARBZ2 = TAR_BZ2 # === TARBZ2

  # ========================================================================= #
  # === TAR_GZ
  # ========================================================================= #
  TAR_GZ  = '.tar.gz'
    TARGZ = TAR_GZ # === TARGZ

  # ========================================================================= #
  # === ZIP
  # ========================================================================= #
  ZIP   = '.zip'

  # ========================================================================= #
  # === GEM
  # ========================================================================= #
  GEM   = '.gem'

  # ========================================================================= #
  # === LZ
  # ========================================================================= #
  LZ    = '.lz'

  # ========================================================================= #
  # === DEFAULT_ARCHIVE_TYPE
  #
  # This constant may be required so that other classes know which is
  # the default archive. This can then be used to, for example, repackage
  # archives into this format.
  # ========================================================================= #
  DEFAULT_ARCHIVE_TYPE = TAR_XZ # Default is .tar.xz for now.

  # ========================================================================= #
  # === ARRAY_REGISTERED_ARCHIVES
  #
  # Keep these archive formats registered in this Array.
  # ========================================================================= #
  ARRAY_REGISTERED_ARCHIVES = [
    TAR_XZ,
    TAR_BZ2,
    TAR_GZ,
    ZIP,
    GEM,
    LZ
  ]

  # ========================================================================= #
  # === RUBY_SRC_DIR
  #
  # This deliberately hard-coded constant is for my home system - it
  # helps speed up the development process if I can use such constants
  # as shortcuts.
  # ========================================================================= #
  RUBY_SRC_DIR = "#{HOME_FOR_X}programming/ruby/src/"
    RUBY_SRC_DIR_AT_HOME = RUBY_SRC_DIR # === RUBY_SRC_DIR_AT_HOME
    RUBY_SRC_DIRECTORY   = RUBY_SRC_DIR # === RUBY_SRC_DIRECTORY
    RUBY_SRC             = RUBY_SRC_DIR # === RUBY_SRC

  # ========================================================================= #
  # === FILE_USE_FLAGS
  # ========================================================================= #
  FILE_USE_FLAGS = "#{project_yaml_directory}use_flags/use_flags.yml"

  # ========================================================================== #
  # === FILE_COOKBOOK_CHAINED_PROGRAMS
  #
  # Easier constant to keep track of the chained cookbook-programs. In
  # this context, a chain means a successive run of programs that are
  # to be compiled.
  #
  # This will then yield to a path such as the following:
  #
  #   /usr/lib/ruby/site_ruby/3.0.0/rbt/yaml/chained_programs/chained_programs.yml
  #
  # ========================================================================== #
  FILE_COOKBOOK_CHAINED_PROGRAMS =
    "#{project_yaml_directory}chained_programs/chained_programs.yml"

  # ========================================================================= #
  # === RBT.file_abbreviations
  #
  # Note that this file will store only abbreviations to the programs,
  # not the name of the program at hand.
  #
  # On my home system this file may reside here:
  #
  #   /home/x/programming/ruby/src/rbt/lib/rbt/yaml/abbreviations_to_the_programs/abbreviations_to_the_programs.yml
  #
  # ========================================================================= #
  def self.file_abbreviations(
      project_yaml_directory = RBT.project_yaml_directory?
    )
    "#{project_yaml_directory}"\
    "abbreviations_to_the_programs/abbreviations_to_the_programs.yml"
  end

  # ========================================================================= #
  # === FILE_COOKBOOK_ALIASES
  # ========================================================================= #
  FILE_COOKBOOK_ALIASES =
    "#{project_yaml_directory}cookbook_aliases/cookbook_aliases.yml"

  # ========================================================================= #
  # === FILE_WINDOWS_COOKBOOK
  # ========================================================================= #
  FILE_WINDOWS_COOKBOOK =
    "#{project_yaml_directory}windows_cookbook.yml"

  # ========================================================================= #
  # === FILE_COMPILE_HOOKS
  # ========================================================================= #
  FILE_COMPILE_HOOKS = "#{project_yaml_directory}compile_hooks.yml"

  # ========================================================================= #
  # === FILE_TIME_COMPILATION
  # ========================================================================= #
  FILE_TIME_COMPILATION =
    "#{project_yaml_directory}configuration/time_compilation.yml"

  # ========================================================================= #
  # === FILE_PROGRAMS_DIRECTORY
  # ========================================================================= #
  FILE_PROGRAMS_DIRECTORY =
    "#{project_yaml_directory}configuration/programs_directory.yml"

  # ========================================================================= #
  # === FILE_TRY_TO_USE_THE_FILE_WINDOWS_COOKBOOK
  # ========================================================================= #
  FILE_TRY_TO_USE_THE_FILE_WINDOWS_COOKBOOK = 
    "#{project_yaml_directory}configuration/try_to_use_the_file_windows_cookbook.yml"

  # ========================================================================== #
  # === FILE_USE_MD5SUM
  # ========================================================================== #
  FILE_USE_MD5SUM = "#{project_yaml_directory}configuration/use_md5sum.yml"

  # ========================================================================== #
  # === FILE_USE_PORG
  # ========================================================================== #
  FILE_USE_PORG = "#{project_yaml_directory}configuration/use_porg.yml"

  # ========================================================================== #
  # === FILE_SHORT_TUTORIAL
  # ========================================================================== #
  FILE_SHORT_TUTORIAL =
    "#{RBT.project_base_dir?}tutorial/short_tutorial.md" 

  # ========================================================================== #
  # === RBT.file_short_tutorial?
  # ========================================================================== # 
  def self.file_short_tutorial?
    FILE_SHORT_TUTORIAL
  end

  # ========================================================================= #
  # === RUBY_SRC_DIR_RBT
  #
  # This constant is only useful on my home system.
  #
  # It corresponds to:
  #
  #   /home/x/programming/ruby/src/rbt/lib/rbt/
  #
  # ========================================================================= #
  RUBY_SRC_DIR_RBT = "#{RUBY_SRC_DIR}rbt/lib/rbt/"
    ROEBE_PATH_TO_COOKBOOKS = RUBY_SRC_DIR_RBT # === ROEBE_PATH_TO_COOKBOOKS

  # ========================================================================= #
  # === RUBY_SRC_DIR_RBT_YAML
  #
  # This constant may correspond to the String
  # "/home/x/programming/ruby/src/rbt/lib/rbt/yaml/".
  # ========================================================================= #
  RUBY_SRC_DIR_RBT_YAML = "#{RUBY_SRC_DIR_RBT}yaml/"
    # ======================================================================= #
    # === RUBY_SRC_DIR_RBT_YAML_AT_HOME
    # ======================================================================= #
    RUBY_SRC_DIR_RBT_YAML_AT_HOME = RUBY_SRC_DIR_RBT_YAML

  # ========================================================================= #
  # === RUBY_SRC_RBT_COOKBOOKS
  #
  # This constant refers to the following path on my home system:
  #
  #   /home/x/programming/ruby/src/rbt/lib/rbt/yaml/cookbooks/
  #
  # ========================================================================= #
  RUBY_SRC_RBT_COOKBOOKS = "#{RUBY_SRC_DIR_RBT_YAML}cookbooks/"

  # ========================================================================= #
  # === GOBOLINUX_LAYOUT
  #
  # This constant is probably only needed for gobolinux systems.
  # ========================================================================= #
  GOBOLINUX_LAYOUT =
    "#{ENV['GOBOLINUX_LAYOUT']}" # Is always a String.

  # ========================================================================= #
  # === REGISTER_COMPILED_PROGRAM
  #
  # Whether we should register compiled programs or not.
  # ========================================================================= #
  REGISTER_COMPILED_PROGRAM = true

  # ========================================================================== #
  # === COMPILE_EVERYTHING_INTO_STANDALONE_DIRECTORY
  #
  # Whether to compile everything into a standalone directory or not.
  # ========================================================================== #
  COMPILE_EVERYTHING_INTO_STANDALONE_DIRECTORY = false

  # ========================================================================= #
  # === SHELL_ERROR_CODE
  #
  # Shell Error code - to append. Since as of September 2022 this won't
  # include any ' ' character. Before that it did.
  # ========================================================================= # 
  SHELL_ERROR_CODE = '2>&1'
        ERROR_LINE = SHELL_ERROR_CODE

  # ========================================================================= #
  # === DEFAULT_PROGRAM_ENTRY
  # ========================================================================= #
  DEFAULT_PROGRAM_ENTRY = 'htop'
          DEFAULT_ENTRY = DEFAULT_PROGRAM_ENTRY # an "alias" to the above ^^^

  # ========================================================================= #
  # === FILE_CPUINFO
  # ========================================================================= #
  FILE_CPUINFO = '/proc/cpuinfo'

  # ========================================================================= #
  # === HOST_CPU
  #
  # The host CPU that the scripts are running on.
  # ========================================================================= #
  HOST_CPU   = RbConfig::CONFIG['host_cpu']

  # ========================================================================= #
  # === TARGET_CPU
  #
  # The target CPU that we are running on.
  # ========================================================================= #
  TARGET_CPU = RbConfig::CONFIG['target_cpu']

  # ========================================================================== #
  # === MAKE_COMMAND_TO_USE
  #
  # Which make command to use by default.
  # ========================================================================== #
  MAKE_COMMAND_TO_USE = 'make'

  # ========================================================================== #
  # === CHECK_FOR_BINARY_NAMES
  # ========================================================================== #
  CHECK_FOR_BINARY_NAMES  = true

  # ========================================================================== #
  # === CHECK_FOR_LIBRARY_NAMES
  # ========================================================================== #
  CHECK_FOR_LIBRARY_NAMES = true

  # ========================================================================== #
  # === SHOW_EXTRA_INFORMATION
  # ========================================================================== #
  SHOW_EXTRA_INFORMATION = true

  # ========================================================================= #
  # === LINUX
  # ========================================================================= #
  LINUX = "#{ENV['LINUX']}"

  # ========================================================================= #
  # === DIRECTORY_INDIVIDUAL_COOKBOOKS_AT_ROEBE_HOME
  #
  # This constant is used mostly on my home system.
  # ========================================================================= #
  DIRECTORY_INDIVIDUAL_COOKBOOKS_AT_ROEBE_HOME =
    "#{RUBY_SRC_DIR_RBT_YAML}cookbooks/"

  # ========================================================================= #
  # === COOKBOOKS_BASE_DIRECTORY
  #
  # The following constant is actually only useful for my home system,
  # and only if I don't use any other setting. So this is mostly just
  # here for legacy reasons.
  # ========================================================================= #
  COOKBOOKS_BASE_DIRECTORY =
    "#{HOME_DIRECTORY_FOR_USER_X}programming/ruby/src/"\
    "rbt/lib/rbt/yaml/cookbooks/"

  # ========================================================================= #
  # === RBT_HOMEPAGE_URL
  #
  # The RBT Homepage is defined here.
  # ========================================================================= #
  RBT_HOMEPAGE_URL = 'https://rubygems.org/gems/rbt'

  # ========================================================================= #
  # === DEFAULT_COLOUR_TO_USE
  #
  # Specify what is our main colour to use for the Cookbooks project.
  # ========================================================================= #
  DEFAULT_COLOUR_TO_USE = "\e[0;32m" # This is GREEN, which is defined in module Colours.

  # ========================================================================= #
  # === RBT::ARRAY_HELP_OPTIONS                                    (help tag)
  #
  # All the help options are stored here. These are the ones that
  # work on e. g. "rbt --help".
  # ========================================================================= #
  ARRAY_HELP_OPTIONS = %w(
    HELP -h --help --help? help hel he h
    display_standard_help_options
  )+['Display help']

  # ========================================================================= #
  # === HASH_ENGLISH_NUMBERS
  # ========================================================================= #
  HASH_ENGLISH_NUMBERS = {
    'one'   => 1,
    'two'   => 2,
    'three' => 3,
    'four'  => 4,
    'five'  => 5,
    'six'   => 6,
    'seven' => 7,
    'eight' => 8,
    'nine'  => 9
  }

  # ========================================================================= #
  # === ARRAY_HELP_VERSION_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_VERSION_OPTIONS = %w(
    --version?
    --version
    version?
    VERSION
    version
  )

  # ========================================================================= #
  # === ARRAY_SPECIAL_COMPONENT_CONTAINING_MANY_SUBCOMPONENTS
  #
  # Here we add constants that can be used like in this way:
  #
  #   ry --plasma1
  #   ry --gnome3
  #
  # ========================================================================= #
  ARRAY_SPECIAL_COMPONENT_CONTAINING_MANY_SUBCOMPONENTS = %w(
    kde
    kde_apps
    mate
    game
    xfce
    xorg
    gnome
    plasma
    lxqt
    portingaids
  )

  # ========================================================================= #
  # === KDE_REGEX_FOR_REMOTE_ARCHIVES
  #
  # This is the regex that can be used in order to obtain the proper
  # remote URL to the individual KDE packages.
  #
  # As an example, have a look here:
  #
  #   https://rubular.com/r/3ZX5Rz8soGqjsw
  #
  # ========================================================================= #
  KDE_REGEX_FOR_REMOTE_ARCHIVES =
    /<tr><td valign="top"><a href="(.+.tar.xz)"><img/

  # ========================================================================= #
  # === REGEXES_TO_SHOW_THE_SOURCE_DIRECTORY
  #
  # These regexes can be used to show/check for the source directory.
  # ========================================================================= #
  REGEXES_TO_SHOW_THE_SOURCE_DIRECTORY = [
    /^-?-?source(-|_| )?dir\??$/,
    /^-?-?source(-|_| )?archive\??$/,
    /^-?-?src(-|_| )?dir\??$/
  ]

  # ========================================================================= #
  # === REGEX_TO_SANITIZE_LIBRARIES
  #
  # Use this regex if you wish to sanitize some libraries.
  # ========================================================================= #
  REGEX_TO_SANITIZE_LIBRARIES = /(.+)\.{(\w+),(\w+)}?/

  # ========================================================================= #
  # === REGEX_TO_USE_FOR_FOR_GITHUB
  #
  # This constant is used to grab archives from github.
  #
  # These archives have a form like:
  #
  #   https://github.com/ninja-build/ninja/archive/v1.7.1.tar.gz
  #
  # The following example on rubular shows how to use this:
  #
  #   https://rubular.com/r/f78srzR8v8E0As
  #
  # The match that will be returned is then a String such as:
  #
  #   1.7.1.tar.gz
  #
  # ========================================================================= #
  REGEX_TO_USE_FOR_FOR_GITHUB = /^http.+\/archive\/v(.+)$/

  # ========================================================================= #
  # === RBT.validation_scripts?
  #
  # Simply return the constant here. This will tell us where the validation
  # scripts are stored.
  # ========================================================================= #
  def self.validation_scripts?
    "#{RBT.project_base_directory?}validation/"
  end; self.instance_eval { alias validation_directory? validation_scripts? } # === RBT.validation_directory?
       self.instance_eval { alias validation_directory  validation_scripts? } # === RBT.validation_directory
       self.instance_eval { alias directory_validation? validation_scripts? } # === RBT.directory_validation?

  # ========================================================================= #
  # === validation_scripts
  # ========================================================================= #
  def validation_scripts?
    RBT.validation_scripts?
  end

  # ========================================================================= #
  # === RBT.file_rename_tab_title
  # ========================================================================= #
  def self.file_rename_tab_title(
      use_this_directory = @configuration_directory
    )
    "#{use_this_directory}rename_tab_title.yml"
  end

  # ========================================================================= #
  # === RBT.file_use_colours?
  #
  # This method will point to a path such as:
  #
  #   /Programs/Ruby/2.5.1/lib/ruby/site_ruby/2.5.0/rbt/yaml/configuration/use_colours.yml
  #
  # ========================================================================= #
  def self.file_use_colours?(
      use_this_directory = @configuration_directory
    )
    "#{use_this_directory}use_colours.yml"
  end; self.instance_eval { alias file_use_colours file_use_colours? } # === RBT.file_use_colours

  # ========================================================================= #
  # === RBT.toplevel_methods?
  #
  # Simply return the constant here. This will tell us where the
  # module_method .rb files are kept.
  #
  # This will typically return a path such as:
  #
  #   /Programs/Ruby/2.6.0/lib/ruby/site_ruby/2.6.0/rbt/toplevel_methods/
  #
  # ========================================================================= #
  def self.toplevel_methods?
    "#{RBT.project_base_directory?}toplevel_methods/"
  end; self.instance_eval { alias toplevel_methods_directory? toplevel_methods? } # === RBT.toplevel_methods_directory?

  # ========================================================================= #
  # === toplevel_methods?
  # ========================================================================= #
  def toplevel_methods?
    RBT.toplevel_methods?
  end

  # ========================================================================= #
  # === RBT.file_registered_cookbook_entries
  #
  # Tell use where the file registered_cookbook_entries.yml is stored.
  #
  # On my home system this may return a String such as:
  #
  #   /home/Programs/Ruby/2.7.1/lib/ruby/site_ruby/2.7.0/rbt/yaml/specification/registered_cookbook_entries.yml
  #
  # ========================================================================= #
  def self.file_registered_cookbook_entries(
      use_this_directory = yaml_directory?
    )
    "#{use_this_directory}specification/registered_cookbook_entries.yml"
  end

  # ========================================================================= #
  # === RBT.file_specification_of_registered_cookbook_entries
  # ========================================================================= #
  def self.file_specification_of_registered_cookbook_entries(
      use_this_directory = yaml_directory?
    )
    "#{use_this_directory}specification/specification_of_registered_cookbook_entries.yml"
  end; self.instance_eval { alias return_file_specification_of_registered_cookbook_entries file_specification_of_registered_cookbook_entries } # === RBT.return_file_specification_of_registered_cookbook_entries

  # ========================================================================= #
  # === REGISTERED_COOKBOOK_ENTRIES
  #
  # The following constant contains all registered entries in the cookbook.
  #
  # These can be found specified in the file registered_cookbook_entries.yml.
  #
  # It is an Array.
  #
  # The file itself resides at:
  #
  #   $RSRC/rbt/lib/rbt/yaml/specification/registered_cookbook_entries.yml
  #
  # ========================================================================= #
  file_registered_cookbook_entries = RBT.file_registered_cookbook_entries
  if File.exist? file_registered_cookbook_entries
    REGISTERED_COOKBOOK_ENTRIES =
      YAML.load_file(file_registered_cookbook_entries)
  else
    REGISTERED_COOKBOOK_ENTRIES = []
  end

  # ========================================================================= #
  # === RBT.file_cookbook_aliases
  #
  # Simply return where the cookbook_aliases.yml file resides.
  #
  # This is, for instance, here:
  #
  #   /Programs/Ruby/2.5.1/lib/ruby/site_ruby/2.5.0/rbt/yaml/cookbook_aliases.yml
  #
  # ========================================================================= #
  def self.file_cookbook_aliases
    "#{RBT.project_yaml_directory?}cookbook_aliases/cookbook_aliases.yml"
  end; self.instance_eval { alias file_cookbook_aliases? file_cookbook_aliases } # === RBT.file_cookbook_aliases?

  # ========================================================================= #
  # === RBT.file_ruby_gem_install_options
  # ========================================================================= #
  def self.file_ruby_gem_install_options
    "#{RBT.project_yaml_directory?}ruby_gem_install_options.yml"
  end

  # ========================================================================== #
  # === RBT.file_programs_version
  # ========================================================================== #
  def self.file_programs_version
    "#{RBT.project_yaml_directory?}programs_version/programs_version.yml"
  end

  # ========================================================================= #
  # === RBT.file_debug
  #
  # This method will refer to the debug.yml file.
  # ========================================================================= #
  def self.file_debug
    "#{RBT.configuration_directory?}debug.yml"
  end

  # ========================================================================= #
  # === FILE_CCACHE
  # ========================================================================= #
  FILE_CCACHE = "#{project_yaml_directory}configuration/use_ccache.yml"

  # ========================================================================== #
  # === RBT.file_allowed_tags
  #
  # This method will return the path to the allowed_tags (a yaml file).
  # ========================================================================== #
  def self.file_allowed_tags
    "#{RBT.project_yaml_directory?}specification/allowed_tags.yml"
  end

  # ========================================================================= #
  # === RBT.chained_programs?
  #
  # Simply return the dataset from the yaml file here.
  #
  # The file that is used here is the file chained_programs.yml,
  # which you can access via:
  #
  #   bl $RSRC/rbt/lib/rbt/yaml/chained_programs.yml
  #
  # ========================================================================= #
  def self.chained_programs?(
      i = RBT.file_cookbook_chained_programs
    )
    YAML.load_file(i)
  end; self.instance_eval { alias chained_programs        chained_programs? } # === RBT.chained_programs
       self.instance_eval { alias return_chained_programs chained_programs? } # === RBT.return_chained_programs

  # ========================================================================= #
  # === return_chained_programs
  # ========================================================================= #
  def return_chained_programs
    RBT.chained_programs?
  end

  # ========================================================================= #
  # === RBT.returned_chained_programs_of_the_mate_desktop
  #
  # This will return the mate-desktop entry, in a sanitized manner,
  # in Array form.
  # ========================================================================= #
  def self.returned_chained_programs_of_the_mate_desktop
    result = return_chained_programs['all_of_the_mate_desktop']
    if result.is_a? String
      result = result.delete("\n").delete(' ')
      # ===================================================================== #
      # Turn it into an Array next.
      # ===================================================================== #
      result = result.split(',').flatten
    end
    return result
  end

  # ========================================================================= #
  # === RBT.file_use_yaml_database_for_tags
  # ========================================================================= #
  def self.file_use_yaml_database_for_tags
    "#{RBT.configuration_directory?}use_yaml_database_for_tags.yml"
  end

  # ========================================================================= #
  # === RBT.file_source_directory?
  #
  # This method may return a path such as:
  #
  #   /home/Programs/Ruby/2.7.1/lib/ruby/site_ruby/2.7.0/rbt/yaml/configuration/configuration/source_directory.yml
  #
  # ========================================================================= #
  def self.file_source_directory?
    "#{RBT.configuration_directory?}source_directory.yml"
  end

  # ========================================================================== #
  # === RBT.file_available_programs_versions
  # ========================================================================== #
  def self.file_available_programs_versions
    "#{RBT.project_yaml_directory?}programs_version/available_programs_versions.md"
  end

  # ========================================================================== #
  # === RBT.file_windows_cookbook
  # ========================================================================== #
  def self.file_windows_cookbook
    "#{RBT.project_yaml_directory?}windows/cookbook.yml"
  end

  # ========================================================================== #
  # === RBT.file_installation_procedere
  #
  # This may refer to the path of the file called
  # 'installation_procedere.yml'.
  # ========================================================================== #
  def self.file_installation_procedere
    "#{RBT.project_yaml_directory?}"\
    "installation_procedere/installation_procedere.yml"
  end

  # ========================================================================= #
  # === RBT.file_build_system_priorities
  # ========================================================================= #
  def self.file_build_system_priorities
    "#{RBT.project_yaml_directory?}"\
    "build_systems_priorities/build_systems_priorities.yml"
  end

  # ========================================================================= #
  # === RBT.registered_cookbook_entries?
  #
  # Simply return all registered cookbook entries through this
  # getter method.
  # ========================================================================= #
  def self.registered_cookbook_entries?
    REGISTERED_COOKBOOK_ENTRIES
  end; self.instance_eval { alias registered_cookbook_entries registered_cookbook_entries? } # === RBT.registered_cookbook_entries?

  # ========================================================================= #
  # === FILE_GLOBAL_FLAGS
  # ========================================================================= #
  FILE_GLOBAL_FLAGS =
    "#{project_yaml_directory}global_flags.yml"

  # ========================================================================= #
  # == ROEBE_FILE_GLOBAL_FLAGS
  #
  # This constant here is only for my home system, hence the prefix
  # ROEBE in it.
  # ========================================================================= #
  ROEBE_FILE_GLOBAL_FLAGS =
    "#{RUBY_SRC_DIR_RBT_YAML}"\
    "#{File.basename(FILE_GLOBAL_FLAGS)}"

  # ========================================================================= #
  # === RBT.file_global_flags
  # ========================================================================= #
  def self.file_global_flags
    "#{RBT.project_yaml_directory?}global_flags.yml"
  end

  # ========================================================================= #
  # === SPECIFICATION_OF_REGISTERED_COOKBOOK_ENTRIES
  #
  # This constant here depends on the other constant called
  # FILE_SPECIFICATION_OF_REGISTERED_COOKBOOK_ENTRIES. That
  # constant is defined above.
  # ========================================================================= #
  if File.exist? RBT.file_registered_cookbook_entries
    SPECIFICATION_OF_REGISTERED_COOKBOOK_ENTRIES = YAML.load_file(
      RBT.file_specification_of_registered_cookbook_entries
    )
  else
    SPECIFICATION_OF_REGISTERED_COOKBOOK_ENTRIES = nil
  end

  # ========================================================================= #
  # === RBT.file_base_template_for_cookbook_yaml_files
  #
  # This constant specifies the path to the base-template file that
  # can be used for creating new .yml files, for the RBT::Cookbooks
  # namespace.
  #
  # This used to be the constant FILE_BASE_TEMPLATE_FOR_COOKBOOK_YAML_FILES
  # until August 2022.
  # ========================================================================= #
  def self.file_base_template_for_cookbook_yaml_files
    "#{RBT.project_base_directory?}cookbooks/"\
    "template/base_template_for_cookbook_yaml_files.md"
  end

  # ========================================================================= #
  # === DIRECTORY_RBT_PROFILES
  #
  # This may be a path such as:
  #
  #   /home/Programs/Ruby/2.6.5/lib/ruby/site_ruby/2.6.0/rbt/misc/profiles/
  #
  # ========================================================================= #
  DIRECTORY_RBT_PROFILES =
    "#{RBT.project_base_directory?}misc/profiles/"
    DIRECTORY_TO_THE_PROFILES = DIRECTORY_RBT_PROFILES # === DIRECTORY_TO_THE_PROFILES

  # ========================================================================= #
  # === RBT.file_programs_directory
  # ========================================================================= #
  def self.file_programs_directory
    "#{RBT.project_yaml_directory?}configuration/programs_directory.yml"
  end

  # ========================================================================== #
  # === FILE_REGISTERED_HEADERS
  #
  # Prefer to use the method RBT.file_registered_headers instead - this
  # constant is mildly deprecated (but still available if you really
  # need it).
  # ========================================================================== #
  FILE_REGISTERED_HEADERS =
    "#{project_yaml_directory}registered/headers.yml"

  # ========================================================================= #
  # === RBT.file_registered_headers
  # ========================================================================= #
  def self.file_registered_headers
    "#{RBT.project_yaml_directory?}registered/headers.yml"
  end

  # ========================================================================= #
  # === RBT.file_available_programs
  # ========================================================================= #
  def self.file_available_programs
    "#{RBT.project_yaml_directory?}available_programs/available_programs.yml"
  end

  # ========================================================================= #
  # === RBT.directory_expanded_cookbooks?
  #
  # On my home system, this method will return the following String:
  #
  #   "/home/Temp/rbt/expanded_cookbooks/"
  #
  # ========================================================================= #
  def self.directory_expanded_cookbooks?
    "#{RBT.rbt_log_directory?}expanded_cookbooks/"
  end; self.instance_eval { alias directory_expanded_cookbooks          directory_expanded_cookbooks? } # === RBT.directory_expanded_cookbooks
       self.instance_eval { alias base_dir_to_store_expanded_cookbooks? directory_expanded_cookbooks? } # === RBT.base_dir_to_store_expanded_cookbooks?
       self.instance_eval { alias expanded_cookbooks?                   directory_expanded_cookbooks? } # === RBT.expanded_cookbooks?
       self.instance_eval { alias path_to_expanded_directory?           directory_expanded_cookbooks? } # === RBT.path_to_expanded_directory?
       self.instance_eval { alias expanded_cookbook_directory?          directory_expanded_cookbooks? } # === RBT.expanded_cookbook_directory?

  # ========================================================================== #
  # === RBT.file_cookbook_chained_programs
  #
  # This is the preferred way to determine where the file
  # "cookbook_chained_programs.yml" can be found.
  #
  # On my home system in April 2021, this yielded the following path:
  #
  #   /usr/lib/ruby/site_ruby/3.0.0/rbt/yaml/chained_programs/chained_programs.yml
  #
  # ========================================================================== #
  def self.file_cookbook_chained_programs
    "#{RBT.project_yaml_directory?}chained_programs/chained_programs.yml"
  end; self.instance_eval { alias file_chained_programs file_cookbook_chained_programs } # === RBT.file_chained_programs

  # ========================================================================= #
  # === RBT.directory_yaml_expanded_cookbooks
  #
  # On my home system, this may yield:
  #
  #   /usr/lib/ruby/site_ruby/3.0.0/rbt/yaml/expanded_cookbooks/
  #
  # ========================================================================= #
  def self.directory_yaml_expanded_cookbooks
    "#{RBT.project_yaml_directory?}expanded_cookbooks/"
  end

  # ========================================================================= #
  # === registered_cookbook_entries?
  #
  # Just a pointer to the module-method with the same name.
  # ========================================================================= #
  def registered_cookbook_entries?
    RBT.registered_cookbook_entries?
  end

  # ======================================================================= #
  # === CHAINED_PROGRAMS
  # ======================================================================= #
  _ = RBT.file_cookbook_chained_programs
  if File.exist? _
    CHAINED_PROGRAMS = YAML.load_file(_)
    # ===================================================================== #
    # Add a few aliases to CHAINED_PROGRAMS
    # ===================================================================== #
    CHAINED_PROGRAMS['xorg1'] = CHAINED_PROGRAMS['xorg_protos']
    CHAINED_PROGRAMS['xorg2'] = CHAINED_PROGRAMS['xorg_utils']
    CHAINED_PROGRAMS['xorg3'] = CHAINED_PROGRAMS['xorg_libraries']
    CHAINED_PROGRAMS['xorg4'] = CHAINED_PROGRAMS['xorg_data']
    CHAINED_PROGRAMS['xorg5'] = CHAINED_PROGRAMS['xorg_apps']
    CHAINED_PROGRAMS['xorg6'] = CHAINED_PROGRAMS['xorg_fonts']
    CHAINED_PROGRAMS['xorg7'] = CHAINED_PROGRAMS['xorg_server']
  else
    CHAINED_PROGRAMS = {} # Use an empty hash in this case.
    puts "The file at `#{RBT.file_cookbook_chained_programs}` does not exist."
  end

  # ========================================================================= #
  # === RBT.utility_scripts?
  #
  # This method will return the path to our utility_scripts/ directory.
  # This is the directory where all utility scripts are stored.
  # ========================================================================= #
  def self.utility_scripts?
    "#{RBT.project_base_directory?}utility_scripts/"
  end; self.instance_eval { alias utility_directory  utility_scripts? } # === RBT.utility_directory
       self.instance_eval { alias utility_directory? utility_scripts? } # === RBT.utility_directory?

  # ========================================================================= #
  # === utility_scripts
  # ========================================================================= #
  def utility_scripts?
    RBT.utility_scripts?
  end

  # =========================================================================== #
  # === RBT.yaml_directory_for_the_registered_yaml_files?
  # =========================================================================== #
  def self.yaml_directory_for_the_registered_yaml_files?
    "#{RBT.yaml_directory?}registered/"
  end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  require 'colours'
  include RBT
  e "#{Colours.rev}Testing the rbt-constants next:"
  e
  pp ARRAY_REGISTERED_ARCHIVES
  e
  e 'Next some constants:'
  e
  e RBT::RUBY_SRC_DIR
  e RBT::RUBY_SRC_DIR_RBT
  e RBT::RUBY_SRC_DIR_RBT_YAML
  e
  e RBT::RUBY_SRC_DIR
  e
  begin
  e 'Sysbin directory is at: '+RBT.sysbin_directory?
  rescue NoMethodError => error; pp error; end
  e
  e 'The validation scripts can be found here:'
  e
  e "  #{RBT.validation_scripts?}"
  e
  e 'The configuration directory can be found at: '+
     RBT.configuration_directory?
  e
  # ========================================================================= #
  # Output the registered cookbook entries next:
  # ========================================================================= #
  pp RBT.registered_cookbook_entries?
  e
  e 'The tutorial could be found here: '+
     RBT.file_short_tutorial?
  e
  e 'The file programs version can be found at: '+
     RBT.file_programs_version
  e
  # pp RBT::CHAINED_PROGRAMS
  # e
  # The next one outputs too much, IMO.
  # pp RBT.chained_programs?
  e
  e 'The following programs constitute the mate-desktop:'
  e
  pp RBT.returned_chained_programs_of_the_mate_desktop
  e
  e RBT::FILE_COOKBOOK_CHAINED_PROGRAMS
  e
  e RBT::SPECIFICATION_OF_REGISTERED_COOKBOOK_ENTRIES if RBT::SPECIFICATION_OF_REGISTERED_COOKBOOK_ENTRIES
  e
  e RBT.file_cookbook_aliases
  e
  e RBT.file_available_programs
  e
  e RBT.file_cookbook_chained_programs
  e
  e RBT.directory_yaml_expanded_cookbooks
  e
end
