#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ImproveTheCookbookRecipes
#
# Note that this class should only be used on my home system - I use it
# to improve the cookbook files, before they are then published on
# rubygems.org. Other users don't really have a need to use this class.
#
# The primary change was to improve the "keep_extracted" entry, so this
# class is no longer necessary - but it will be retained, in the event
# that we may have to change something in regards to the individual .yml
# files in the future. It is thus a "batch-sanitizer" of some sorts.
#
# Note that this class is mostly only useful on my home system, so it
# uses hardcoded paths.
#
# Usage example:
#
#   RBT::ImproveTheCookbookRecipes.new(ARGV)
#
# =========================================================================== #
# require 'rbt/cookbooks/improve_the_cookbook_recipes/improve_the_cookbook_recipes.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class ImproveTheCookbookRecipes < Base # === RBT::ImproveTheCookbookRecipes

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === verbose_report_to_the_user_that_no_problem_was_found
  # ========================================================================= #
  def verbose_report_to_the_user_that_no_problem_was_found
    opne 'No problem was found. This is great! '+
         gold(cheering_person)
  end

  # ========================================================================= #
  # === this_has_been_corrected
  # ========================================================================= #
  def this_has_been_corrected
    e "#{rev}This has been corrected."
  end

  # ========================================================================= #
  # === set_work_on_these_yaml_files
  # ========================================================================= #
  def set_work_on_these_yaml_files(i)
    @work_on_these_yaml_files = i
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    set_work_on_these_yaml_files(
      Dir[RUBY_SRC_DIR+'rbt/lib/rbt/yaml/cookbooks/*.yml']
    )
    found_n_problems = 0 # How many faulty .yml files were found.
    @work_on_these_yaml_files.each {|entry|
      dataset = File.read(entry) # Read in the full content of the .yml file here.
      # ===================================================================== #
      # === keep_extracted: f
      # ===================================================================== #
      if dataset =~ /keep_extracted: f$/
        found_n_problems += 1
        e 'The file `'+steelblue(entry.to_s)+rev+
          '` has a less-than-optimal '\
          'keep_extracted value (of "f").'
        if is_on_roebe?
          this_has_been_corrected
          # copy_file(entry, '/Depot/j/'+File.basename(entry))
          data = File.read(entry)
          data.sub!(/keep_extracted: f/,'keep_extracted: no')
          e 'Saving into '+sfile(entry)+rev+' next.'
          write_what_into(data, entry)
        end
      end
      # ===================================================================== #
      # === keep_extracted: t
      # ===================================================================== #
      if dataset =~ /keep_extracted: t$/
        found_n_problems += 1
        e 'The file `'+steelblue(entry.to_s)+'` has a less-than-optimal '\
          'keep_extracted value (of "t").'
        if is_on_roebe?
          this_has_been_corrected
          data = File.read(entry)
          data.sub!(/keep_extracted: t/,'keep_extracted: yes')
          e 'Saving into '+sfile(entry)+rev+' next.'
          write_what_into(data, entry)
        end
      end
      # ===================================================================== #
      # === symlink_headers: t
      # ===================================================================== #
      if dataset =~ /symlink_headers: t$/
        found_n_problems += 1
        e 'The file `'+steelblue(entry.to_s)+'` has a less-than-optimal '\
          'symlink_headers value (of "t").'
        if is_on_roebe?
          this_has_been_corrected
          data = File.read(entry)
          data.sub!(/symlink_headers: t/,'symlink_headers: yes')
          e 'Saving into '+sfile(entry)+rev+' next.'
          write_what_into(data, entry)
        end
      end
      # ===================================================================== #
      # === symlink_pkgconfig_files: t
      # ===================================================================== #
      if dataset =~ /symlink_pkgconfig_files: t$/
        found_n_problems += 1
        e 'The file `'+steelblue(entry.to_s)+'` has a less-than-optimal '\
          'symlink_pkgconfig_files value (of "t").'
        if is_on_roebe?
          this_has_been_corrected
          data = File.read(entry)
          data.sub!(/symlink_pkgconfig_files: t/,'symlink_pkgconfig_files: yes')
          e 'Saving into '+sfile(entry)+rev+' next.'
          write_what_into(data, entry)
        end
      end
    }
    if found_n_problems == 0
      verbose_report_to_the_user_that_no_problem_was_found
    end
  end

  # ========================================================================= #
  # === RBT::ImproveTheCookbookRecipes[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::ImproveTheCookbookRecipes.new(ARGV)
end # improvethecookbookrecipes