#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::CreateBigYamlCookbook
#
# This class will create one huge .yaml file which holds all the various
# yaml datasets.
#
# Usage example:
#
#   RBT::Cookbooks::CreateBigYamlCookbook.new(ARGV)
#
# =========================================================================== #
# require 'rbt/generate_files/create_big_yaml_cookbook.rb'
# RBT::Cookbooks::CreateBigYamlCookbook.new(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class CreateBigYamlCookbook < Base # === RBT::Cookbooks::CreateBigYamlCookbook

  require 'rbt/requires/require_the_cookbook_class.rb'

  # ========================================================================= #
  # === STORE_INTO_THIS_FILE
  # ========================================================================= #
  STORE_INTO_THIS_FILE =
    "#{RBT.rbt_log_directory?}big_yaml_cookbook.yml"

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @hash
    # ======================================================================= #
    @hash = {} # This is the main Hash.
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === create_big_yaml_file
  # ========================================================================= #
  def create_big_yaml_file
    _ = available_programs?
    _.each {|program|
      opne simp(program)
      hash = {}
      dataset = action(:SanitizeCookbook).return_array(program)
      hash[program] = dataset
      @hash.update(hash)
    }
    what = YAML.dump(@hash)
    into = STORE_INTO_THIS_FILE
    write_what_into(what, into)
    opne "Storing into `#{sfile(into)}`."
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    opne 'Now creating a big yaml file. This will take some time.'
    create_big_yaml_file
  end

  # ========================================================================= #
  # === RBT::Cookbooks::CreateBigYamlCookbook[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::CreateBigYamlCookbook.new(ARGV)
end # bigyamlcookbook