#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::GUI::UniversalWidgets::UrlAction
#
# Usage example:
#
#   RBT::GUI::UniversalWidgets::UrlAction.new(ARGV)
#
# =========================================================================== #
# require 'rbt/gui/universal_widgets/url_action/url_action.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module GUI

module UniversalWidgets

class UrlAction < ::RBT::Base # === RBT::GUI::UniversalWidgets::UrlAction

  require 'universal_widgets/base_module/base_module.rb'
  include ::UniversalWidgets::BaseModule

  require 'rbt/url_action/url_action.rb'

  # ========================================================================= #
  # === USE_THESE_CSS_RULES
  # ========================================================================= #
  USE_THESE_CSS_RULES = '
  '

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'URL action'

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH = '82% or minimum 1280px'

  # ========================================================================= #
  # === HEIGHT
  # ========================================================================= #
  HEIGHT = '40% or minimum 300px'

  # ========================================================================= #
  # === USE_THIS_FONT
  # ========================================================================= #
  USE_THIS_FONT = :dejavu_condensed_20

  # ========================================================================= #
  # === LARGER_FONT
  # ========================================================================= #
  LARGER_FONT   = :dejavu_condensed_22

  # ========================================================================= #
  # === SMALLER_FONT
  # ========================================================================= #
  SMALLER_FONT  = :dejavu_condensed_18

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments       = ARGV,
      run_already                 = true
    )
    super(:vertical) if use_gtk3?
    determine_the_GUI_to_be_used(commandline_arguments) # This must come first, even before reset().
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    on_delete_event_quit_the_application
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super() if respond_to?(:super)
    reset_the_shared_module # This can come early.
    reset_the_base_module
    reset_the_internal_variables
    infer_the_namespace
    # ======================================================================= #
    # === @configuration
    # ======================================================================= #
    @configuration = [true, __dir__, namespace?]
    # ======================================================================= #
    # === Set the title, width, height and the font in use.
    # ======================================================================= #
    title_width_height_font(TITLE, WIDTH, HEIGHT, USE_THIS_FONT)
    if use_gtk3?
      handle_CSS_rules
    end
    infer_the_size_automatically
    # ======================================================================= #
    # === @url_action
    # ======================================================================= #
    @url_action = ::RBT::UrlAction.new('', :do_not_run_yet)
  end

  # ========================================================================= #
  # === favicon?
  # ========================================================================= #
  def favicon?
    :tabble
  end

  # ========================================================================= #
  # === smaller_font?
  # ========================================================================= #
  def smaller_font?
    SMALLER_FONT
  end

  # ========================================================================= #
  # === handle_CSS_rules                                            (CSS tag)
  # ========================================================================= #
  def handle_CSS_rules
    use_gtk_paradise_project_css_file
    append_project_css_file
    more_CSS_then_apply_it(USE_THESE_CSS_RULES)
  end

  # ========================================================================= #
  # === reset_the_shared_module
  #
  # This method can be used for ruby-gtk3 and ruby-libui, among other
  # toolkits.
  # ========================================================================= #
  def reset_the_shared_module
  end

  # ========================================================================= #
  # === send_message_to_url_action
  # ========================================================================= # 
  def send_message_to_url_action(
      this_action = 'query/wget'
    )
    @url_action.set_action(this_action)
    @url_action.run
    set_buffer_text(@url_action.result)
  end  

  # ========================================================================= #
  # === create_the_entries
  # ========================================================================= #
  def create_the_entries
    # ========================================================================== #
    # === @entry
    # ========================================================================== #
    @entry = create_entry
    @entry.set_max_length 50
    # @entry.set_activates_default(true)
    @entry.signal_connect(:key_press_event) { |widget, event|
      key_pressed = Gdk::Keyval.to_name(event.keyval)
      case key_pressed
      when 'Return','KP_Enter'
        @entry.set_focus true
        @button.signal_emit :clicked
      else # debug info
        # pp key_pressed
      end
    }
  end

  # ========================================================================= #
  # === create_scrolled_windows
  # ========================================================================= #
  def create_scrolled_windows
    @source_buffer = create_text_buffer
    @source_view   = create_text_view(@source_buffer) 
  end

  # ========================================================================= #
  # === send_message_to_url_action_from_the_main_entry
  # ========================================================================= #
  def send_message_to_url_action_from_the_main_entry
    send_message_to_url_action(@entry.text)
  end

  # ========================================================================= #
  # === create_the_buttons                                         (buttons tag)
  # ========================================================================= #
  def create_the_buttons
    # ======================================================================= #
    # === @button
    # ======================================================================= #
    @button = button('_Send Message', self, :use_mnemonics) {
      :send_message_to_url_action_from_the_main_entry
    }
    @button.clear_background
    @button.on_hover(:lightgreen)
    @button.pad8px
    @button.bblack2  
  end

  # ========================================================================= #
  # === set_buffer_text
  # ========================================================================= #
  def set_buffer_text(i)
    @source_buffer.text = i
  end

  # ========================================================================= #
  # === padding?
  # ========================================================================= #
  def padding?
    4
  end

  # ========================================================================= #
  # === border_size?
  # ========================================================================= #
  def border_size?
    2
  end

  # ========================================================================= #
  # === create_the_skeleton                        (create tag, skeleton tag)
  # ========================================================================= #
  def create_the_skeleton
    create_scrolled_windows
    create_the_buttons
    create_the_entries
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    run_super
  end

  # ========================================================================= #
  # === connect_the_skeleton                                    (connect tag)
  # ========================================================================= #
  def connect_the_skeleton
    abort_on_exception

    outer_vbox = create_vbox # A new vbox.

    outer_vbox.maximal(@entry,  4)
    mini_hbox = create_hbox
    mini_hbox.minimal(@button)
    outer_vbox.minimal(mini_hbox, 4)
    outer_vbox.add(@source_view)

    window = create_window_or_runner
    window << outer_vbox

    properly_prepare_this_window(window,
      {
        title:       title?,
        font:        font?,
        width:       width?,
        height:      height?,
        padding:     padding?,
        border_size: border_size?
      }
    )
    window.show_all
    window.top_left
    do_all_startup_related_actions
    run_main
  end

  # ========================================================================= #
  # === do_all_startup_related_actions
  # ========================================================================= #
  def do_all_startup_related_actions
    Thread.new {
      sleep 0.0001
    }
  end

  # ========================================================================= #
  # === RBT::GUI::Gtk::UrlAction.run
  # ========================================================================= #
  def self.run(
      i = ARGV
    )
    require 'gtk_paradise/run'
    _ = ::RBT::GUI::Gtk::UrlAction.new(i)
    r = ::Gtk.run
    _.set_parent_widget(r) # Must come before we enable the key-combinations.
    r << _
    r.add_shortcut(1,  'focus_on_entry', :alt)
    r.add_shortcut(:a, 'focus_on_entry', :alt)
    r.automatic_size_then_automatic_title
    r.enable_quick_exit
    r.set_background :white
    r.top_left_then_run
  end; self.instance_eval { alias start_gui_application run } # === RBT::GUI::Gtk::UrlAction.start_gui_application

  # ========================================================================= #
  # === RBT::GUI::UniversalWidgets::UrlAction[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::GUI::UniversalWidgets::UrlAction.new(ARGV)
end # gui_rbt