#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# The job of this file is to return all programs that are available
# in the homebrew project. 
# =========================================================================== #
# require 'rbt/homebrew/registered_programs_of_homebrew.rb'
# =========================================================================== #
module RBT

  require 'rbt/toplevel_methods/e_and_esystem.rb'
  require 'rbt/colours/colours.rb'

  # ========================================================================= #
  # === RBT::REMOTE_URL_TO_THE_HOMEBREW_FORMULAS
  # ========================================================================= #
  REMOTE_URL_TO_THE_HOMEBREW_FORMULAS =
    'https://formulae.brew.sh/formula/'

  # ========================================================================= #
  # === RBT.return_all_registered_programs_of_homebrew
  # ========================================================================= #
  def self.return_all_registered_programs_of_homebrew
    require 'rbt/check_for_updates/base.rb'
    remote_url = REMOTE_URL_TO_THE_HOMEBREW_FORMULAS
    e "#{rev}Next checking the remote URL #{sfancy(remote_url)}#{rev}."
    _ = RBT::Cookbooks::CheckForRemoteWebpages.new
    _.obtain_dataset_from_the_remote_webpage(
      remote_url
    )
    result = _.dataset?
    # ======================================================================= #
    # The following regex-check can be seen here:
    #
    #   https://rubular.com/r/5b7Tsl61E6ovGO
    #
    # It has two matches.
    # ======================================================================= #
    use_this_regex = 
      /<td><a href="\/formula\/([A-Za-z0-9]+)">.+<\/a>/
    scanned = result.scan(use_this_regex).flatten.compact
    scanned
  end

  # ========================================================================= #
  # === RBT.report_all_registered_programs_of_homebrew
  # ========================================================================= #
  def self.report_all_registered_programs_of_homebrew
    these_entries = RBT.return_all_registered_programs_of_homebrew
    e 'The following entries were found:'
    index = 0
    these_entries.each {|name_of_the_program, program_version| index += 1
      padded_index = (index.to_s.rjust(4)).ljust(6)
      colourized_and_padded_index = RBT.dodgerblue(padded_index)
      e '  '+colourized_and_padded_index+
        name_of_the_program.delete('-').ljust(40)+
        program_version
    }
  end

  # ========================================================================= #
  # === RBT.return_programs_that_are_exlusively_registered_in_the_homebrew_programs
  #
  # Usage example:
  #
  #   x = RBT.return_programs_that_are_exlusively_registered_in_the_homebrew_programs
  #
  # ========================================================================= #
  def self.return_programs_that_are_exlusively_registered_in_the_homebrew_programs
    require 'rbt/toplevel_methods/available_programs.rb'
    # ======================================================================= #
    # Obtain all available programs next.
    # ======================================================================= #
    available_programs = RBT.available_programs?
    # ======================================================================= #
    # Obtain all programs available in the homebrew "registry". We will
    # query this "registry" by parsing the homepage.
    # ======================================================================= #
    remote_registered_programs = RBT.return_all_registered_programs_of_homebrew.map {|entry|
      entry.delete('-_')
    } # On 25.08.2022 this Array had 4618 programs registered.
    remote_registered_programs.reject! {|entry|
      available_programs.include?(entry)
    } # On 25.08.2022, the unique entries at homebrew included
      # 3699 programs not registered in the RBT suite.
    return remote_registered_programs
  end

end

if __FILE__ == $PROGRAM_NAME
  require 'colours/colours_e_autoinclude.rb'
  # RBT.return_all_registered_programs_of_homebrew
  # RBT.report_all_registered_programs_of_homebrew
  x = RBT.return_programs_that_are_exlusively_registered_in_the_homebrew_programs
  e
  pp x
  e
  e rev+'There are a total of '+
    rev+
    sfancy(RBT.return_all_registered_programs_of_homebrew.size.to_s)+
    rev+' programs available in homebrew.'
  e "#{rev}Of these, #{steelblue(x.size.to_s)}"\
    "#{rev} are #{swarn('not')}#{rev} registered in the rbt gem."
end # rbtregisteredprogramsofhomebrew