#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Linux::RenameSystemMap
#
# This .rb file will rename a file called 'System.map' to the properly
# named kernel version, such as 'System.map-5.3.12'.
#
# Usage example:
#
#   RBT::Linux::RenameSystemMap.new
#
# =========================================================================== #
# require 'rbt/linux/kernel/rename_system_map.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Linux

class RenameSystemMap < ::RBT::Base # === RBT::Linux::RenameSystemMap

  # ========================================================================= #
  # === USR_SRC_LINUX
  # ========================================================================= #
  USR_SRC_LINUX = '/usr/src/linux'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already           = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @kernel_version
    # ======================================================================= #
    @kernel_version = '5.4.31'
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    _ = 'System.map'
    if File.exist? _
      # ===================================================================== #
      # First we try to determine whether we have a readlink from which
      # we can infer the kernel version.
      # ===================================================================== #
      if File.symlink?(USR_SRC_LINUX)
        @kernel_version = File.readlink(USR_SRC_LINUX).split('-').last.strip
      end
      new_name = "#{_}-#{@kernel_version}"
      if File.exist? new_name
        opne "Can not rename #{sfile(new_name)} as it already exists"
        opne 'at '+steelblue("/boot/#{new_name}")
      else
        opne "Renaming the file #{sfile(_)} to #{sfile(new_name)}."
        rename_file(_, new_name)
      end
    else
      no_file_exists_at _
    end
  end

  # ========================================================================= #
  # === RBT::Linux::RenameSystemMap[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Linux::RenameSystemMap.new
end # renamesystemmap