#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::UpdateSlackpkg
#
# This small script can be used to quickly update the slackpage version
# that is in use.
#
# Usage example:
#
#   RBT::UpdateSlackpkg.new
#
# =========================================================================== #
# require 'rbt/linux/slackware/update_slackpkg.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class UpdateSlackpkg < Base # === RBT::UpdateSlackpkg

  # ========================================================================= #
  # === URL
  #
  # This URL is hardcoded. Perhaps in the future we may transition into
  # a dynamic-query instead.
  #
  # For the FTP listing of most recent version of slackpkg, try 
  # this:
  #
  #   https://slackpkg.org/stable/?C=M;O=D
  #
  # ========================================================================= #
  URL = 'https://slackpkg.org/stable/slackpkg-2.83.0-noarch-4.txz'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === url?
  # ========================================================================= #
  def url?
    URL
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    e 'Trying to update the slackpkg version (on Slackware) next.'
    url = url?
    wget(url)
    target = File.basename(url)
    esystem "installpkg #{target}"
  end

  # ========================================================================= #
  # === UpdateSlackpkg[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::UpdateSlackpkg.new(ARGV)
end # updateslackpkg