#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CreateLogFile
#
# This class shall eventually log which environment variables were
# used for the compilation process. It is, however had, currently
# (May 2020) unfinished.
#
# Usage example:
#
#   RBT::CreateLogFile.new
#
# =========================================================================== #
# require 'rbt/logging/create_log_file.rb'
# RBT::CreateLogFile.new(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class CreateLogFile < Base # === RBT::CreateLogFile

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      parse_this_input(yielded)
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @hash
    # ======================================================================= #
    @hash = {}
    # ======================================================================= #
    # === :installation_was_a_success_or_failure
    #
    # Populate the hash with some dummy-variables on startup:
    # ======================================================================= #
    @hash[:installation_was_a_success_or_failure] = :failure
    # ======================================================================= #
    # === :program_name
    # ======================================================================= #
    @hash[:program_name] = :foobar
    # ======================================================================= #
    # === :full_configure_command
    # ======================================================================= #
    @hash[:full_configure_command] = '../configure --prefix=/usr'
    # ======================================================================= #
    # === :store_where
    # ======================================================================= #
    @hash[:store_where] = "#{program_name?}.yml"
    # ======================================================================= #
    # === :uses_appdir_prefix
    # ======================================================================= #
    @hash[:uses_appdir_prefix] = false
    # ======================================================================= #
    # === :use_environment_flags
    # ======================================================================= #
    @hash[:use_environment_flags] = false
  end

  # ========================================================================= #
  # === program_name?
  # ========================================================================= #
  def program_name?
    @hash[:program_name].to_s
  end

  # ========================================================================= #
  # === clear_the_main_hash
  # ========================================================================= #
  def clear_the_main_hash
    @hash.clear
  end

  # ========================================================================= #
  # === uses_appdir_prefix?
  # ========================================================================= #
  def uses_appdir_prefix?
    @hash[:uses_appdir_prefix]
  end

  # ========================================================================= #
  # === set_program_name
  # ========================================================================= #
  def set_program_name(i)
    @hash[:program_name] = i.to_sym
  end

  # ========================================================================= #
  # === parse_this_input
  # ========================================================================= #
  def parse_this_input(i)
    if i.is_a? Hash
      # ===================================================================== #
      # === :clear_the_main_hash
      # ===================================================================== #
      if i.has_key? :clear_the_main_hash
        clear_the_main_hash if (i.delete(:clear_the_main_hash) == true)
      end
      # ===================================================================== #
      # === :use_environment_flags
      # ===================================================================== #
      if i.has_key? :use_environment_flags
        @hash[:use_environment_flags] = i.delete(:use_environment_flags)
      end
      # ===================================================================== #
      # === :program_name
      # ===================================================================== #
      if i.has_key? :program_name
        @hash[:program_name] = i.delete(:program_name)
      end
      # ===================================================================== #
      # === :uses_appdir_prefix
      # ===================================================================== #
      if i.has_key? :uses_appdir_prefix
        @hash[:uses_appdir_prefix] = i.delete(:uses_appdir_prefix)
      end
      # ===================================================================== #
      # === :store_where
      # ===================================================================== #
      if i.has_key? :store_where
        @hash[:store_where] = i.delete(:store_where)
      end
    end
  end

  # ========================================================================= #
  # === store_where?
  # ========================================================================= #
  def store_where?
    @hash[:store_where]
  end

  # ========================================================================= #
  # === store_file
  # ========================================================================= #
  def store_file(
      store_where = store_where?
    )
    _ = @hash
    opne "#{rev}Storing into the file `#{sfile(store_where)}#{rev}`."
    # ======================================================================= #
    # Handle environment-flags part first:
    # ======================================================================= #
    if _.has_key?(:use_environment_flags) and (_.delete(:use_environment_flags) == true)
      #_.delete(:use_environment_flags)
      write_what_into(return_the_dataset_for_the_environment, store_where)
    else
      _ = YAML.dump(_)
      write_what_into(_, store_where)
    end
  end

  # ========================================================================= #
  # === return_the_dataset_for_the_environment
  # ========================================================================= #
  def return_the_dataset_for_the_environment
    "Time of installation:  #{dd_mm_yyyy}, #{hh_mm_ss}#{N}"\
    "PATH:                  #{ENV['PATH']}#{N}"\
    "CFLAGS:                #{ENV['CFLAGS']}#{N}"
  end

  # ========================================================================= #
  # === do_store_into_the_appdir
  # ========================================================================= #
  def do_store_into_the_appdir(store_here)
    opne "#{rev}Storing into the file `#{sfile(store_here)}#{rev}`."
    write_what_into(
      return_the_dataset_for_the_environment,
      store_here
    )
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    store_file
  end

  # ========================================================================= #
  # === RBT::CreateLogFile[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::CreateLogFile.new(ARGV)
end # createlogfile