#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# It is recommended to use the method calls rather than the constants,
# as the method-call may be a bit more dynamic, and thus flexible.
# =========================================================================== #
# require 'rbt/project/project.rb'
# =========================================================================== #
module RBT

  # ========================================================================= #
  # === RBT::PROJECT_BASE_DIRECTORY
  # ========================================================================= #
  PROJECT_BASE_DIRECTORY = File.absolute_path("#{__dir__}/..")+'/'

  # ========================================================================= #
  # === RBT.project_base_directory?
  #
  # This method will yield to us the base directory of the RBT project.
  #
  # For example, it will return a String such as:
  #
  #   /Programs/Ruby/2.6.0/lib/ruby/site_ruby/2.6.0/rbt/
  #   /home/Programs/Ruby/2.7.1/lib/ruby/site_ruby/2.7.0/rbt/
  #
  # It may be another path on your computer system, such as
  # /usr/lib/ruby/site_ruby/2.7.0/rbt/.
  # ========================================================================= #
  def self.project_base_directory?
    RBT::PROJECT_BASE_DIRECTORY
  end; self.instance_eval { alias project_base_dir? project_base_directory? } # === RBT.project_base_dir?
       self.instance_eval { alias project_dir?      project_base_directory? } # === RBT.project_dir?
       self.instance_eval { alias base_dir?         project_base_directory? } # === RBT.base_dir?
       self.instance_eval { alias base_directory?   project_base_directory? } # === RBT.base_directory?

  # ========================================================================= #
  # === RBT::PROJECT_YAML_DIRECTORY
  #
  # This constant specifies where our project-specific YAML files
  # are to be kept - the yaml base directory.
  #
  # There are several "aliases" to this constant here.
  #
  # This constant will store a path, as String, such as this one here:
  #
  #   /Programs/Ruby/2.5.1/lib/ruby/site_ruby/2.5.0/rbt/yaml/
  #
  # In February 2024 this constant was removed - use the method
  # RBT.project_yaml_directory? instead, if you need to find out
  # where the yaml directory is kept at.
  # ========================================================================= #
  # PROJECT_YAML_DIRECTORY = "#{project_base_dir?}yaml/"

  # ========================================================================= #
  # === project_dir?
  # ========================================================================= #
  def project_dir?
    RBT.project_base_dir?
  end

  # ========================================================================= #
  # === RBT.project_yaml_dir?
  #
  # This is a query-method for the constant PROJECT_YAML_DIRECTORY.
  # ========================================================================= #
  def self.project_yaml_dir?
    "#{project_base_dir?}yaml/"
  end; self.instance_eval { alias yaml_directory?           project_yaml_dir? } # === RBT.yaml_directory?
       self.instance_eval { alias project_yaml_directory?   project_yaml_dir? } # === RBT.project_yaml_directory?
       self.instance_eval { alias cookbooks_yaml_directory? project_yaml_dir? } # === RBT.cookbooks_yaml_directory?

  # ========================================================================= #
  # === rbt_yaml_directory?
  # ========================================================================= #
  def rbt_yaml_directory?
    self.project_yaml_directory?
  end; alias cookbooks_yaml_directory? rbt_yaml_directory? # === cookbooks_yaml_directory?

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  e RBT.project_base_dir?
  e RBT.project_yaml_dir?
end