#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::Registered
#
# This class could be used as a base class for RegisteredBinaries and
# RegisteredLibraries.
# =========================================================================== #
# require 'rbt/registered/registered.rb'
# RBT.yaml_directory_for_the_registered_yaml_files?
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class Registered < Base # === RBT::Cookbooks::Registered

  require 'rbt/toplevel_methods/return_version_of_this_program.rb'

  # ========================================================================= #
  # === SHOW_PROGRAM_NAME
  #
  # If true then we will show the program in question.
  # ========================================================================= #
  SHOW_PROGRAM_NAME = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_args = ARGV
    )
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === set_this_program
  # ========================================================================= #
  def set_this_program(i)
    @this_program = i
  end

  # ========================================================================= #
  # === this_program?
  # ========================================================================= #
  def this_program?
    @this_program
  end; alias input? this_program? # === input?

  # ========================================================================= #
  # === also_storing_into
  # ========================================================================= #
  def also_storing_into
    e "#{rev}Also storing into:"
  end

  # ========================================================================= #
  # === hash_to_be_stored?
  #
  # This is our main dataset.
  # ========================================================================= #
  def hash_to_be_stored?
    @hash_to_be_stored
  end; alias hash? hash_to_be_stored? # === hash?
       alias hash  hash_to_be_stored? # === hash

  # ========================================================================= #
  # === ruby_src_dir?
  #
  # This refers to the ruby files on my home directory; it's not useful
  # for other users of this project.
  # ========================================================================= #
  def ruby_src_dir?
    RUBY_SRC_DIR_AT_HOME
  end

  # ========================================================================= #
  # === ruby_src_dir_for_the_registered_yaml_files?
  #
  # This is specifically for my home directory, the registered/
  # subdirectory.
  # ========================================================================= #
  def ruby_src_dir_for_the_registered_yaml_files?
    "#{ruby_src_dir?}rbt/lib/rbt/yaml/registered/"
  end

  # ========================================================================= #
  # === it_is_part_of_the_package
  # ========================================================================= #
  def it_is_part_of_the_package(i)
    opne "#{rev}It is part of the package → `#{simp(i)}#{rev}`."
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::Registered.new(ARGV)
end