#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/time/time.rb'
# =========================================================================== #
module RBT

  require 'time'

  # ========================================================================== #
  # === RBT.return_date
  #
  # This will return a date such as:
  #
  #   "20.05.2020"
  #
  # ========================================================================== #
  def self.return_date
    ::Time.now.strftime('%d.%m.%Y')
  end; self.instance_eval { alias today? return_date } # === RBT.today?

  # ========================================================================= #
  # === RBT.return_time
  #
  # This method will return the current time, in hours:minutes::seconds
  # such as "10:11:45".
  # ========================================================================= #
  def self.return_time
    ::Time.now.strftime('%H:%M:%S')
  end

  # ========================================================================= #
  # === RBT.get_date
  #
  # As of Sep 2014, this method will return a string such as:
  #
  #  "12 Jan 2015"
  #
  # ========================================================================= #
  def self.get_date
    current_day = Time.now.strftime('%d') # This will return a string such as "30".
    current_day+
    ' '+
    Date::MONTHNAMES[Date.today.month][0,3]+ # Only obtain the first three letters.
    ' '+
    ::Time.now.strftime('%Y')
  end

  # ========================================================================= #
  # === RBT.get_extended_date
  #
  # This method is similar to RBT.get_date, but we will not truncate the
  # name of the month. So we may return a String such as "12 January 2023".
  # ========================================================================= #
  def self.get_extended_date
    current_day   = Time.now.strftime('%d') # This will return a string such as "30".
    current_month = Date::MONTHNAMES[Date.today.month]
    return "#{current_day}"\
           " #{current_month}"\
           " #{::Time.now.strftime('%Y')}"
  end

end

if __FILE__ == $PROGRAM_NAME
  pp RBT.get_date
end