#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/all_binaries.rb'
# =========================================================================== #
module RBT

  require 'rbt/project/project.rb'
  require 'rbt/requires/require_yaml.rb'
  require 'rbt/constants/constants.rb'

  # ========================================================================= #
  # === FILE_REGISTERED_BINARIES
  #
  # This may be a file path such as:
  #
  #   /home/Programs/Ruby/2.7.1/lib/ruby/site_ruby/2.7.0/rbt/yaml/registered/binaries.yml
  #
  # cat $COOKBOOKS/yaml/registered_binaries.yml
  # ========================================================================= #
  FILE_REGISTERED_BINARIES =
    "#{RBT.yaml_directory_for_the_registered_yaml_files?}binaries.yml"

  # ========================================================================= #
  # === YAML_FILE_REGISTERED_BINARIES
  #
  # This is equivalent to:
  #
  #   data = YAML.load_file("#{RBT.yaml_directory_for_the_registered_yaml_files?}binaries.yml")
  #
  # ========================================================================= #
  if File.exist? FILE_REGISTERED_BINARIES
    YAML_FILE_REGISTERED_BINARIES = YAML.load_file(FILE_REGISTERED_BINARIES)
  else
    YAML_FILE_REGISTERED_BINARIES = {} # Otherwise, initialize as empty hash.
  end; ALL_BINARIES = YAML_FILE_REGISTERED_BINARIES # === ALL_BINARIES

  # ========================================================================= #
  # === RBT.all_binaries?
  #
  # Get back all registered binaries.
  #
  # This is guaranteed to return a Hash.
  # ========================================================================= #
  def self.all_binaries?
    ALL_BINARIES
  end; self.instance_eval { alias all_binaries      all_binaries? } # === RBT.all_binaries
       self.instance_eval { alias hash_all_binaries all_binaries? } # === RBT.hash_all_binaries

  # ========================================================================= #
  # === all_binaries?
  # ========================================================================= #
  def all_binaries?
    RBT.all_binaries?
  end

  # ========================================================================= #
  # === RBT.does_include_this_binary?
  #
  # This will return true if we have a binary with that name.
  #
  # Usage examples:
  #
  #   RBT.does_include_this_binary? 'xxd'
  #   RBT.has_this_binary? 'xzcmp'
  #
  # ========================================================================= #
  def self.does_include_this_binary?(
      i, perform_a_deep_check = false
    )
    all_binaries = all_binaries?
    # ======================================================================= #
    # This entry point is for checking input such as "matescreenshot",
    # which could be "mate-screenshot". Note that as of January 2020
    # this is only enabled for binaries - not sure if we need to enable
    # this for other input-type as well, but binaries make the most
    # sense.
    # ======================================================================= #
    if perform_a_deep_check
      all_binaries.transform_keys! {|entry| entry.delete('-') }
    end
    all_binaries.has_key?(i)
  end; self.instance_eval { alias has_this_binary?             does_include_this_binary? } # === RBT.has_this_binary?
       self.instance_eval { alias is_this_a_registered_binary? does_include_this_binary? } # === RBT.is_this_a_registered_binary?

  # ========================================================================= #
  # === RBT.file_registered_binaries
  #
  # This method should be used preferentially, rather than the older
  # constant.
  # ========================================================================= #
  def self.file_registered_binaries
    "#{RBT.yaml_directory_for_the_registered_yaml_files?}binaries.yml"
  end; self.instance_eval { alias location_registered_binaries file_registered_binaries } # === RBT.location_registered_binaries

end

if __FILE__ == $PROGRAM_NAME
  require 'opn'
  require 'colours/colours_e_autoinclude.rb'
  pp RBT.all_binaries?
  e
  e "#{rev}This was read from the file:"
  e
  e ::Colours.sfile("  #{RBT.file_registered_binaries}")
  e
  is_it_included = RBT.does_include_this_binary?('xxd')
  opn; e 'Is `'+sfancy('xxd')+rev+'` a part of the rbt project? '+
         simp(is_it_included.to_s)
  if is_it_included
    opn; e rev+'It belongs to the program called `'+
           sfancy(
             RBT::YAML_FILE_REGISTERED_BINARIES['xxd']
           )+rev+'`.'
  end
  e
end # allbinaries?