#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# The code in this file can be used to check a cookbook .yml file for
# any errors.
# =========================================================================== #
# require 'rbt/toplevel_methods/check_this_cookbook_file_for_errors.rb'
# RBT.check_this_cookbook_file_for_errors
# =========================================================================== #
module RBT

  require 'rbt/toplevel_methods/e_and_esystem.rb'
  require 'rbt/toplevel_methods/opn.rb'
  require 'rbt/constants/constants.rb'
  require 'rbt/colours/colours.rb'
  require 'rbt/requires/require_yaml.rb'

  # ========================================================================= #
  # === RBT.does_this_cookbook_file_have_any_erroneous_entry?
  #
  # This method will return a boolen value - true or false.
  #
  # If true then the cookbook file in question has had an erroneous
  # entry. 
  # ========================================================================= #
  def self.does_this_cookbook_file_have_any_erroneous_entry?(
      this_file = "#{RUBY_SRC_RBT_COOKBOOKS}jasper.yml"
    )
    if this_file.is_a? Array
      this_file = this_file.first
    end
    if this_file.nil?
      this_file = "#{RUBY_SRC_RBT_COOKBOOKS}jasper.yml"
    end
    if File.exist? this_file
      registered_cookbook_entries = RBT.registered_cookbook_entries?
      dataset = YAML.load_file(this_file)
      key = dataset.keys.first
      available_entries_from_the_cookbooks_file = dataset[key].keys
      # ===================================================================== #
      # Next, obtain the entries that were not registered. Keep in mind
      # that this Array may be entry.
      # ===================================================================== #
      these_entries_were_not_registered = available_entries_from_the_cookbooks_file.reject {|entry|
        registered_cookbook_entries.include? entry
      }
      !these_entries_were_not_registered.empty?
    else
      false # You have to ensure that the file exists on your own.
    end
  end

  # ========================================================================= #
  # === RBT.check_this_cookbook_file_for_errors
  #
  # The first argument should be an existing .yml file holding the
  # dataset that may be faulty.
  # ========================================================================= #
  def self.check_this_cookbook_file_for_errors(
      this_file =
        "#{RUBY_SRC_RBT_COOKBOOKS}jasper.yml"
    )
    if this_file.is_a? Array
      this_file = this_file.first
    end
    if this_file.nil?
      this_file = "#{RUBY_SRC_RBT_COOKBOOKS}jasper.yml"
    end
    unless this_file.end_with?('.yml')
      unless File.exist? '.yml'
        this_file = project_yaml_dir?+'cookbooks/'+this_file+'.yml'
      end
    end
    if File.exist? this_file
      registered_cookbook_entries = RBT.registered_cookbook_entries?
      dataset = YAML.load_file(this_file)
      key = dataset.keys.first
      available_entries_from_the_cookbooks_file = dataset[key].keys
      # ===================================================================== #
      # Next, obtain the entries that were not registered. Keep in mind
      # that this Array may be entry.
      # ===================================================================== #
      these_entries_were_not_registered = available_entries_from_the_cookbooks_file.reject {|entry|
        registered_cookbook_entries.include? entry
      }
      unless these_entries_were_not_registered.empty?
        opn; e 'The file at `'+sfile(this_file)+'`'
        opn; e 'contains at the least one erroneous entry.'
        these_entries_were_not_registered.each {|this_entry_is_not_included|
          opn; e 'The entry `'+sfancy(this_entry_is_not_included)+rev+
                 '` is not registered.'
        }
      else
        opn; e rev+'Everything is fine for the file '+
               sfile(this_file)+rev+'.'
        opn; e "#{rev}It has #{RBT.violet('no incorrect entries')}."
      end
    else
      opn; e "#{rev}No file exists at #{sfile(this_file)}#{rev}."
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.check_this_cookbook_file_for_errors(ARGV)
end # checkthiscookbookfileforerrors htop.yml