#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/compile_these_programs.rb'
# RBT.set_compile_these_programs
# =========================================================================== #
module RBT

  require 'rbt/constants/constants.rb'
  require 'rbt/toplevel_methods/toplevel_methods.rb'
  require 'rbt/toplevel_methods/try_to_return_a_special_compile_component.rb'
  require 'rbt/actions/individual_actions/filter_for_valid_program_names/filter_for_valid_program_names.rb'

  # ========================================================================= #
  # === @dataset_containing_the_abbreviations
  # ========================================================================= #
  @dataset_containing_the_abbreviations = nil
  if File.exist? RBT.file_abbreviations
    @dataset_containing_the_abbreviations = 
      YAML.load_file(RBT.file_abbreviations)
  end

  # ========================================================================= #
  # === @compile_these_programs
  #
  # This must always be an Array.
  #
  # It denotes which programs the user wishes to compile.
  #
  # It is stored on the toplevel area because it may be re-used across
  # different classes and modules belonging to the RBT namespace.
  # ========================================================================= #
  @compile_these_programs = []

  # ========================================================================= #
  # === RBT.set_compile_these_programs
  #
  # Do note that whenever this method is called, the old value for
  # @compile_these_programs will be overwritten.
  # ========================================================================= #
  def self.set_compile_these_programs(
      i,
      &block
    )
    if i.is_a? Hash
      # ===================================================================== #
      # === :compile_these_programs
      # ===================================================================== #
      if i.has_key? :compile_these_programs
        i = i.delete(:compile_these_programs)
      end
    end
    # ======================================================================= #
    # We need an Array here:
    # ======================================================================= #
    i = [i].flatten.compact
    i.map! {|entry| # Will always be an Array.
      entry = entry.dup if entry.frozen?
      if block_given?
        yielded = yield
        case yielded
        # =================================================================== #
        # === :remove_underscore
        # =================================================================== #
        when :remove_underscore
          entry.delete!('_')
        end
      end
      if entry.is_a? String
        entry = action(:filter_for_valid_program_names,
          [entry, @dataset_containing_the_abbreviations]
        )
      end
      # ===================================================================== #
      # The next line will remove '-' and '_' characters from the given
      # input. This, however had, may NOT be done when there is a
      # locally existing file with that name. On 23.11.2019 this was
      # disabled, because some binaries, such as "wx-config", may
      # include a '-' character.
      # ===================================================================== #
      entry # And return it here too.
    }.compact
    @compile_these_programs = i
  end; self.instance_eval { alias compile_these_programs=    set_compile_these_programs } # === RBT.compile_these_programs=
       self.instance_eval { alias set_compile_this           set_compile_these_programs } # === RBT.set_compile_this
       self.instance_eval { alias set_install_these_programs set_compile_these_programs } # === RBT.set_install_these_programs

  # ========================================================================= #
  # === RBT.set_compile_this_as_first_program
  # ========================================================================= #
  def self.set_compile_this_as_first_program(i)
    @compile_these_programs[0] = i
  end

  # ========================================================================= #
  # === RBT.compile_these_programs?
  # ========================================================================= #
  def self.compile_these_programs?
    @compile_these_programs
  end; self.instance_eval { alias compile_which_programs? compile_these_programs? } # === RBT.compile_which_programs?
       self.instance_eval { alias install_which_programs? compile_these_programs? } # === RBT.install_which_programs?

end

if __FILE__ == $PROGRAM_NAME
  pp RBT.set_compile_these_programs(ARGV)
end # rb compile_these_programs.rb htop php libx11 /home/x/src/kalarm/kalarm-21.12.0.tar.xz