#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/configure_has_a_higher_priority_than_meson.rb'
# =========================================================================== #
module RBT

  # ========================================================================= #
  # The following .rb file contains the location to the file containing
  # the build-systems-priorities.
  # ========================================================================= #
  require 'rbt/constants/constants.rb'
  require 'rbt/requires/require_raw_yaml.rb'

  # ========================================================================= #
  # === RBT.configure_has_a_higher_priority_than_meson?
  #
  # This method will return true if "configure" has a higher priority
  # than "meson". A yaml file keeps track of these priorities,
  # so if you wish to change the priority, you have to modify the
  # content of that .yml file.
  #
  # The name of that yaml file:
  #
  #   build_systems_priorities.yml
  #
  # ========================================================================= #
  def self.configure_has_a_higher_priority_than_meson?(
      this_file = RBT.file_build_system_priorities
    )
    array = YAML.load_file(this_file)
    array.index('configure') < array.index('meson')
  end

end

if __FILE__ == $PROGRAM_NAME
  puts RBT.configure_has_a_higher_priority_than_meson?  
end # configure_has_a_higher_priority_than_meson?