#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/generate_programs_version_yaml_file.rb'
# RBT.generate_programs_version_yaml_file
# =========================================================================== #
module RBT

  require 'rbt/constants/constants.rb'
  require 'rbt/toplevel_methods/roebe.rb'
  require 'rbt/toplevel_methods/e_and_esystem.rb'
  require 'rbt/colours/rev_simp_sdir_sfancy_sfile_and_swarn.rb'
  require 'rbt/toplevel_methods/opn.rb'
  require 'rbt/toplevel_methods/files_and_directories_related_methods.rb'

  # ========================================================================= #
  # === RBT.generate_programs_version_yaml_file
  #
  # This method will create one big yaml file that is a Hash. This
  # big Hash will have the name of all programs and the corresponding
  # version of said programs.
  #
  # The key will be the name of the program and the value will
  # be the respective program version.
  #
  # You can also pass in your own dataset. If you do so, remember to
  # use a hash, with specific key->value pairs that reflect the
  # name of the program and the specific version of said program.
  #
  # You can invoke this method from the commandline, by issuing:
  #
  #   rbt --generate_version_file
  #
  # ========================================================================= #
  def self.generate_programs_version_yaml_file(
      use_this_dataset = nil
    )
    _ = ''.dup # We will append onto this String.
    unless use_this_dataset # This is the default behaviour/clause.
      hash = {} # We store into this hash here.
      target = RUBY_SRC_DIR_RBT_YAML+
               'programs_version/available_programs_versions.md'
      if is_on_roebe?
        e 'Reading in the dataset from this file:'
        e
        e sfile("  #{target}")
        e
      end
      unless File.exist? target
        e 'No file was found at '+sfile(target)+'.'
        exit
      end
      dataset = File.readlines(target)
      dataset.each {|line|
        splitted = line.split(' ').map(&:strip)
        opn; e "Now working on the program "\
               "`#{Colours::HtmlColoursMethods.lightgray(splitted[0])}`."
        # =================================================================== #
        # Add onto the Hash next. Should keep it as a String.
        # =================================================================== #
        hash[splitted[0].downcase] = splitted[1].to_s
      }
      use_this_dataset = hash
    end
    use_this_dataset.each_pair {|key, value|
      value = '"'+value.to_s+'"' unless value.include? '"'
      _ << key.to_s+': '+value.to_s+N
    }
    # ======================================================================= #
    # Ok, now we should have all programs and the respective version
    # at hand - we can store it next. In the past, this made use of
    # YAML.dump() but for some strange reason, YAML.dump() gave strange
    # results; which is why I decided to generate the .yml file
    # on my own.
    # ======================================================================= #
    what = _
    into = "#{RBT.yaml_directory?}programs_version/programs_version.yml"
    opn; e "Will also store into `#{sfile(into)}#{rev}`."
    write_what_into(what, into) # Store into the file programs_version.yml
    # ======================================================================= #
    # Store for my home-system as well.
    # ======================================================================= #
    if is_roebe? # On my system.
      into = "#{RUBY_SRC_DIR_RBT}yaml/programs_version/programs_version.yml"
      opn; e "Will also store into `#{sfile(into)}#{rev}`."
      write_what_into(what, into)
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.generate_programs_version_yaml_file
end # generateprogramsversionyamlfile