#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/home_directory.rb'
# =========================================================================== #
module RBT

  # ========================================================================= #
  # === RBT.home_directory?
  #
  # This should default towards /root or the user's home directory.
  #
  # For example, for the superuser on a system, this will expand into
  # "/root/" or "/home/debug/".
  #
  # The method will ensure that a trailing '/' is part of the result.
  # ========================================================================= #
  def self.home_directory?
    ("#{ENV['HOME']}/").squeeze('/')
  end; self.instance_eval { alias home_dir? home_directory? } # === RBT.home_dir?

  # ========================================================================== #
  # === home_dir?
  # ========================================================================== #
  def home_dir?
    self.home_dir?
  end; alias home_directory? home_dir? # === home_directory?

  # ========================================================================= #
  # === RBT.last_downloaded_file
  # ========================================================================= #
  def self.last_downloaded_file
    "#{RBT.home_dir?}LAST_DOWNLOADED_FILE.md"
  end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  e
  e 'The home directory for this user is: '
  e
  e "  #{RBT.home_directory?}"
  e
  e 'The last downloaded file can be found at:'
  e
  e "  #{RBT.last_downloaded_file}"
  e
end