#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/make_app_prefix.rb'
# =========================================================================== #
module RBT

  require 'rbt/toplevel_methods/e_and_esystem.rb'
  require 'rbt/toplevel_methods/files_and_directories_related_methods.rb'

  # ========================================================================= #
  # === RBT.make_app_prefix
  #
  # This method will ultimately execute a command such as this one
  # here:
  #
  #   make PREFIX=/home/Programs/Xfce4-panel/4.14.3/ install
  #
  # ========================================================================= #
  def self.make_app_prefix(
      for_this_program = :infer
    )
    begin
      require 'program_information'
    rescue LoadError; end
    if for_this_program.is_a? Array and for_this_program.empty?
      for_this_program = :default
    end
    case for_this_program
    # ======================================================================= #
    # === :infer
    #
    # This is the default entry point to this method.
    # ======================================================================= #
    when :infer,
         :default
      _ = File.basename(return_pwd)
      program_name    = ProgramInformation.return_name(_)
      program_version = ProgramInformation.return_version(_)
      target = programs_dir?+
               program_name.capitalize+'/'+
               program_version.to_s+'/'
      # esystem 'make PREFIX='+target
      cmd_to_run = "make PREFIX=#{target} install"
      e
      esystem cmd_to_run
      e
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.make_app_prefix(ARGV)
end # makeappprefix