#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/remove_file_extension.rb'
# =========================================================================== #
module RBT

  # ========================================================================= #
  # === RBT.remove_file_extension
  #
  # This method will try to remove anything we declare to be an "improper
  # file extension". In other words this method will remove the
  # trailing archive type of a file, such as .tar.xz or .tar.gz
  # and so forth.
  #
  # The order is important here, in particular for entries such
  # as "foobar-1.0.tar.xz" or "foobar-1.0.tar.gz".
  #
  # File.basename() is used to not have to worry about leading
  # components, or '/' entries.
  # ========================================================================= #
  def self.remove_file_extension(i)
    i = i.dup if i.frozen?
    if i
      # i = File.basename(i.to_s) # This was disabled in May 2023.
      i = i.sub(/\.bz2$/,  ''). # Handle .tar.bz2 entries.
            sub(/\.gz$/,   ''). # Remove it before .tar.gz entries.
            sub(/\.zip$/,  '').
            sub(/\.har$/,  ''). # Java .jar files.
            sub(/\.tbz$/,  '').
            sub(/\.xz$/,   '').
            sub(/\.lz$/,   ''). # In particular to handle .tar.lz entries.
            sub(/\.Z$/,    '').
            sub(/\.7z$/,   '').
            sub(/\.tar$/,  '').
            sub(/\.js$/,   ''). # Javascript .js files.
            sub(/\.tgz$/,  '').
            sub(/\.tbz2$/, '').
            sub(/\.gzip$/, '').
            sub(/\.phar$/, '').
            sub(/\.txz$/,  '').
            sub(/\.gem$/,  ''). # Handle ruby .gem files here.
            sub(/\.yml$/,  '').
            sub(/\.js$/,   '').
            sub(/-trunk/,  ''). # ← Also remove weird things like that.
            sub(/-source/, '').  # ← and also this.
            sub(/&type=zip$/,'')
    end
    return i # Last but not least, always return it.
  end; self.instance_eval { alias remove_archive_from_the_end remove_file_extension } # === RBT.remove_archive_from_the_end 
       self.instance_eval { alias remove_file_suffix          remove_file_extension } # === RBT.remove_file_suffix
       self.instance_eval { alias no_archive                  remove_file_extension } # === RBT.no_archive

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  require 'colours/html_colours'
  # ========================================================================= #
  # The code that comes next is simply meant for testing the above 
  # checks, in the method.
  # ========================================================================= #
  e
  e '  '+Colours.steelblue(RBT.remove_archive_from_the_end('foobar-1.2.3.tar.xz'))
  e '  '+Colours.steelblue(RBT.remove_archive_from_the_end('setuptools-47.1.1.zip'))
  e '  '+Colours.steelblue(RBT.remove_archive_from_the_end('yada-5.4.3.tar.gz'))
  e '  '+Colours.steelblue(RBT.remove_archive_from_the_end('yada-5.4.3.gzip'))
  e '  '+Colours.steelblue(RBT.remove_archive_from_the_end('subtle-0.11.3224-xi.tbz2'))
  e '  '+Colours.steelblue(RBT.remove_file_extension('glib-2.35.3.tar.xz')) # => "glib-2.35.3"
  e
end # removearchivefromtheend
    # rbtremovefileextension