#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/report_extended_information_about_this_program.rb'
# =========================================================================== #
module RBT

  require 'rbt/requires/require_the_cookbook_class.rb'
  require 'rbt/toplevel_methods/opn.rb'
  require 'rbt/toplevel_methods/cliner.rb'

  # ========================================================================= #
  # === RBT.check_and_report_whether_file_exists
  #
  # The first argument to this method must be the path to the archive.
  # ========================================================================= #
  def self.check_and_report_whether_file_exists(
      i,
      optional_cookbook_object = nil
    )
    i = i.to_s
    unless File.exist? i
      opnn; Colours.ewarn 'This archive does not exist locally, at'
      opnn; Colours.ewarn "  `#{Colours.sfile(i)}#{swarn('`')}"
      if optional_cookbook_object
        if optional_cookbook_object.do_not_download?
          opne 'This is, however had, no problem as it will not'
          opne 'be downloaded anyway.'
        end
      end
      cliner :lightblue
    end
  end

  # ========================================================================= #
  # === RBT.report_extended_information_about_this_program
  #
  # (feedback tag, debug tag, main tag)
  #
  # Use this feedback-method to report extended information about a
  # program, in a colourful manner. This program must have been registered
  # in the Cookbooks namespace - otherwise we can not really display
  # much about that program at hand.
  #
  # The input to this method can be either be a String or an Array.
  # ========================================================================= #
  def self.report_extended_information_about_this_program(
      i
    )
    commandline_arguments = []
    if i.is_a? Array
      commandline_arguments << i.select {|line| line.start_with? '--'}
      i = i.first
    end
    commandline_arguments.flatten!
    commandline_arguments.compact!
    if i
      # ===================================================================== #
      # Delegate towards the sanitize_cookbook action next:
      # ===================================================================== #
      _ = action(:sanitize_cookbook, :do_not_run_yet) { :fast }
      unless commandline_arguments.empty?
        _.menu(commandline_arguments) # Pass the commandline arguments here.
      end
      _.find_this_program(i)
      _.report
      check_and_report_whether_file_exists(
        _.full_path?, _
      )
    end
  end; self.instance_eval { alias feedback_colourized_result report_extended_information_about_this_program } # === RBT.feedback_colourized_result

end

if __FILE__ == $PROGRAM_NAME
  RBT.report_extended_information_about_this_program(ARGV)  
end # scookie htop --no-colours