#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/run_make_then_make_install.rb'
# RBT.run_make_then_make_install
# =========================================================================== #
module RBT

  require 'rbt/toplevel_methods/e_and_esystem.rb'
  require 'rbt/toplevel_methods/files_and_directories_related_methods.rb'
  require 'rbt/colours/rev_simp_sdir_sfancy_sfile_and_swarn.rb'

  # ========================================================================= #
  # === RBT.run_make_then_make_install
  #
  # This method essentially runs "make", followed by "make install".
  #
  # It was created not only in order to chain these two commands,
  # but to also check for a directory called "BUILD_DIRECTORY" in
  # the current working directory. If such a directory exists then
  # we will cd into that directory before running "make" and "make
  # install".
  # ========================================================================= #
  def self.run_make_then_make_install
    _ = 'BUILD_DIRECTORY/'
    e; e; e
    e "#{Colours.rev}Starting with 'make', then 'make install'."
    e; e
    if File.directory? _
      e
      e "#{rev}Changing directory into #{sdir(_)} #{rev}now."
      e
      cd _
    end
    use_this_command = 'make'
    # ======================================================================= #
    # Normally we have to run 'make', but meson-systems require
    # ninja - thus we check for that next.
    # ======================================================================= #
    if File.exist? 'Makefile'
      # ===================================================================== #
      # In this case we will use the above 'make' command.
      # ===================================================================== #
    elsif File.exist? 'build.ninja'
      use_this_command = 'ninja'
    end
    esystem "#{use_this_command}"
    esystem "#{use_this_command} install"
    e; e; e
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.run_make_then_make_install
end # runmakethenmakeinstall