#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/try_to_return_a_special_compile_component.rb'
# =========================================================================== #
module RBT

  require 'rbt/toplevel_methods/try_to_find_shortcut_for_a_chain_compiled_program.rb'

  require 'rbt/toplevel_methods/games.rb'
  # ========================================================================= #
  # === RBT.return_this_game_component
  #
  # The input to this method should typically be a number, such as
  # 1 (or its String-variant "1").
  #
  # Invocation example from the commandline:
  #
  #   ry --game1
  #   ry --game4
  #
  # ========================================================================= #
  def self.return_this_game_component(i)
    result = nil
    big_array = return_games_components
    big_array.flatten!
    if i.to_s.include? '..'
      splitted = i.split('..')
      first_number = splitted.first.to_i - 1
      last_number  = splitted.last.to_i - 1
      last_number  = (big_array.size - 1) if last_number > (big_array.size)
      result = big_array[first_number, last_number]
    else
      # ===================================================================== #
      # Prevent too high numbers next:
      # ===================================================================== #
      i = i.to_i # ← We need Integers past this point.
      i = (big_array.size - 1) if i > (big_array.size - 1)
      result = big_array[i - 1] # Find the proper result
    end
    return result
  end; self.instance_eval { alias try_to_compile_this_game_component         return_this_game_component } # === RBT.try_to_compile_this_game_component
       self.instance_eval { alias try_to_compile_this_game_desktop_component return_this_game_component } # === RBT.try_to_compile_this_game_desktop_component

  # ========================================================================= #
  # === RBT.return_kde5_applications
  #
  # This method will conveniently return the Array holding all programs
  # that belong to the KDE5 Application pool.
  #
  # As of 11.01.2020 there were 224 registered KDE5 applications in the
  # RBT project.
  #
  # This can be determined via the following method:
  #
  #   RBT.return_kde5_applications.size
  #
  # ========================================================================= #
  def self.return_kde5_applications
    result = return_chained_programs['kde5_applications']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end; self.instance_eval { alias return_kde_apps return_kde5_applications } # === RBT.return_kde_apps

  # ========================================================================= #
  # === RBT.return_python_addons
  # ========================================================================= #
  def self.return_python_addons
    result = return_chained_programs['python_addons']
    result = result.split(',') if result.is_a?(String) and result.include?(',')
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === RBT.return_xfce_components
  #
  # This method will conveniently return the Array holding all programs
  # that belong to the KDE5 Application pool.
  # ========================================================================= #
  def self.return_xfce_components(i = 'xfce')
    result = return_chained_programs[i]
    result = result.split(',') if result.is_a?(String) and result.include?(',')
    result.map!(&:strip)
    return result
  end; self.instance_eval { alias return_xfce                    return_xfce_components } # === RBT.return_xfce
       self.instance_eval { alias return_xfce_desktop_components return_xfce_components } # === RBT.return_xfce_desktop_components

  # ========================================================================= #
  # === RBT.return_this_xfce_component
  #
  # The input to this method should typically be a number, such as
  # 1 (or its String-variant "1").
  #
  # Invocation example from the commandline:
  #
  #   ry --xfce1
  #
  # ========================================================================= #
  def self.return_this_xfce_component(i)
  result = nil
    big_array = return_xfce_components
    big_array.flatten!
    if i.to_s.include? '..'
      splitted = i.split('..')
      first_number = splitted.first.to_i - 1
      last_number  = splitted.last.to_i - 1
      last_number  = (big_array.size - 1) if last_number > (big_array.size)
      result = big_array[first_number, last_number]
    else
      # ===================================================================== #
      # Prevent too high numbers next:
      # ===================================================================== #
      i = i.to_i # ← We need Integers past this point.
      i = (big_array.size - 1) if i > (big_array.size - 1)
      result = big_array[i - 1] # Find the proper result
    end
    result
  end; self.instance_eval { alias try_to_compile_this_xfce_component         return_this_xfce_component } # === RBT.try_to_compile_this_xfce_component
       self.instance_eval { alias try_to_compile_this_xfce_desktop_component return_this_xfce_component } # === RBT.try_to_compile_this_xfce_desktop_component

  # ========================================================================= #
  # === RBT.return_mate_desktop_components
  #
  # This method will conveniently return the Array holding all programs
  # that belong to the KDE5 Application pool.
  #
  # Usage example:
  #
  #   RBT.return_mate_desktop_components # => ["matedesktop", "libmatemixer", "libmateweather", etc...
  #
  # ========================================================================= #
  def self.return_mate_desktop_components
    result = return_chained_programs['all_of_the_mate_desktop']
    result = result.split(',') if result.is_a?(String) and result.include?(',')
    result.map!(&:strip)
    return result
  end; self.instance_eval { alias return_mate_applications return_mate_desktop_components } # === RBT.return_mate_applications

  # ========================================================================= #
  # === RBT.return_this_mate_component
  #
  # The input to this method should typically be a number, such as
  # 1 (or its String-variant "1").
  #
  # Invocation example from the commandline:
  #
  #   ry --mate3
  #
  # On my home system I can also use aliases such as these:
  #
  #   mate1
  #   mate2
  #   mate3
  #
  # And so forth.
  # ========================================================================= #
  def self.return_this_mate_component(
      i = 1
    )
    result = nil
    big_array = return_mate_desktop_components
    big_array.flatten!
    if i.to_s.include? '..'
      # ===================================================================== #
      # Handle pseudo-ranges here:
      # ===================================================================== #
      splitted = i.split('..')
      first_number = splitted.first.to_i - 1
      last_number  = splitted.last.to_i - 1
      last_number  = (big_array.size - 1) if last_number > (big_array.size)
      result = big_array[first_number, last_number]
    else
      # ===================================================================== #
      # Prevent too high numbers next:
      # ===================================================================== #
      i = i.to_i # ← We need Integers past this point.
      i = (big_array.size - 1) if i > (big_array.size - 1)
      result = big_array[i - 1] # Find the proper result, by deducting one.
    end
    return result
  end; self.instance_eval { alias try_to_compile_this_mate_component         return_this_mate_component } # === RBT.try_to_compile_this_mate_component
       self.instance_eval { alias try_to_compile_this_mate_desktop_component return_this_mate_component } # === RBT.try_to_compile_this_mate_desktop_component

  # ========================================================================= #
  # === RBT.return_xorg_server
  # ========================================================================= #
  def self.return_xorg_server
    result = return_chained_programs['xorg_server']
    result = result.split(',') if result.include? ','
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === RBT.return_xorg_components
  #
  # This method will conveniently return the Array holding all programs
  # that belong to the KDE5 Application pool.
  # ========================================================================= #
  def self.return_xorg_components
    array  = []
    array << return_xorg_data
    array << return_xorg_libraries
    array << return_xorg_utils
    array << return_xorg_apps
    array << return_xorg_fonts
    array << return_xorg_server_and_post_xorg_server
    array.flatten!
    return array
  end; self.instance_eval { alias return_xorg return_xorg_components } # === RBT.return_xorg

  # ========================================================================= #
  # === RBT.try_to_return_a_special_compile_component
  #
  # This method will take an input, such as "--kde1", and return an
  # associated program, if possible.
  #
  # The reason why this method exists is so that we can use input such
  # as "compile kde1" and just compile the first kde component that
  # is registered. It is thus a convenient shortcut - we don't have to
  # remember any names if we don't want to.
  # ========================================================================= #
  def self.try_to_return_a_special_compile_component(i)
    case i # case tag, menu tag
    # ======================================================================= #
    # === Compile a specific GNOME component
    #
    # This entry point allows us to use numbers in order to return
    # a particular gnome component, and then proceed to compile it.
    # So for example, "gnome1" will compile the first gnome component,
    # "gnome2" will refer to the second gnome component and so forth.
    #
    # Invocation examples:
    #
    #   ry gnome1
    #   ry --gnome2
    #   ry gnome1..20
    #
    # ======================================================================= #
    when /^-?-?gnome(\d{1,3}\.{0,2}\d{0,3})$/ # ← See: http://rubular.com/r/n2r8DwcTnU
      i = return_this_gnome_component($1.to_s.dup)
    # ======================================================================= #
    # === Compile a specific mate-desktop component
    #
    # This entry point allows us to use numbers in order to compile
    # a particular mate-desktop component. So for example, "mate1"
    # will be a shortcut for "matedesktop" whereas "mate2" will
    # be a shortcut for "libmatemixer", and so forth.
    #
    # Invocation examples:
    #
    #   ry mate1
    #   ry mate2
    #   ry mate3
    #   ry --mate1
    #   ry --mate2
    #   ry --mate3
    #   ry mate11
    #   ry mate1..100
    #
    # ======================================================================= #
    when /^-?-?mate(\d{1,3}\.{0,2}\d{0,3})$/ # ← See: https://rubular.com/r/n2r8DwcTnU
      match = $1.to_s.dup
      i = return_this_mate_component(match)
    # ======================================================================= #
    # === Compile a specific KDE5 plasma component
    #
    # This entry point allows us to use numbers in order to compile
    # a particular KDE5 Plasma component.
    #
    # Invocation examples:
    #
    #   ry plasma1
    #   ry --plasma3
    #
    # ======================================================================= #
    when /^-?-?plasma(\d{1,3}\.{0,2}\d{0,3})$/ # ← See: https://rubular.com/r/YQtQZjKtBsmKuj
      match = $1.to_s.dup
      i = return_this_plasma_component(match)
    # ======================================================================= #
    # === Compile a specific KDE5 application
    #
    # Invocation examples:
    #
    #   ry --kde_apps1
    #   ry --kde_apps4
    #   ry kde_apps4
    #
    # ======================================================================= #
    when /^-?-?kde(-|_| )?apps(\d+)$/i
      index = $2.to_s.to_i - 1
      i = return_kde5_applications[index]
    # ======================================================================= #
    # === Compile a specific LXQT-desktop component
    #
    # This entry point allows us to use numbers in order to compile
    # a particular lx-qt-desktop component. So for example, "lxqt1"
    # will be a shortcut for "libqtxdg" whereas "lxqt2" will
    # be a shortcut for "lxqtbuildtools", and so forth.
    #
    # Invocation examples:
    #
    #   ry lxqt1
    #   ry lxqt2
    #   ry lxqt3
    #   ry --lxqt1
    #   ry --lxqt2
    #   ry --lxqt3
    #   ry lxqt11
    #   ry lxqt..100
    #
    # ======================================================================= #
    when /^-?-?lxqt(\d{1,3}\.{0,2}\d{0,3})$/ # ← See: http://rubular.com/r/n2r8DwcTnU
      match = $1.to_s.dup
      i = return_this_lxqt_component(match)
    # ======================================================================= #
    # === Compile a KDE5 porting aids component
    #
    # Invocation examples:
    #
    #   ry portingaids1
    #   ry portingaids2
    #   ry portingaids3
    #   ry portingaids4
    #   ry portingaids5
    #   ry portingaids6
    #
    # ======================================================================= #
    when /^-?-?portingaids(\d{1,3}\.{0,2}\d{0,3})$/ # ← See: http://rubular.com/r/n2r8DwcTnU
      match = $1.to_s.dup
      i = return_this_portingaids_component(match)
    # ======================================================================= #
    # === Compile a specific XORG component
    #
    # This entry point allows us to use numbers in order to compile
    # a particular xorg component. So for example, "xorg1" will compile
    # the first xorg component, "xorg2" will refer to the second
    # xorg component and so forth.
    #
    # Invocation examples:
    #
    #   ry xorg1
    #   ry --xorg2
    #   ry --xorg15
    #   ry xorg1..20
    #
    # ======================================================================= #
    when /^-?-?xorg(\d{1,3}\.{0,2}\d{0,3})$/ # ← See: http://rubular.com/r/n2r8DwcTnU
      match = $1.to_s.dup
      i = return_this_xorg_component(match)
    # ======================================================================= #
    # === Compile a specific XFCE component
    #
    # This entry point allows us to use numbers in order to compile
    # a particular xfce component. So for example, "xfce1" will compile
    # the first xfce component, "xfce2" will refer to the second
    # xfce component and so forth.
    #
    # Invocation examples:
    #
    #   ry xfce1
    #   ry --xfce2
    #   ry xfce1..20
    #
    # ======================================================================= #
    when /^-?-?xfce(\d{1,3}\.{0,2}\d{0,3})$/ # ← See: http://rubular.com/r/n2r8DwcTnU
      match = $1.to_s.dup
      i = return_this_xfce_component(match)
    # ======================================================================= #
    # === Compile a specific game
    #
    # This entry point allows us to use numbers in order to compile
    # a particular mate-desktop component. So for example, "game1"
    # will compile the first game, "game2" will refer to the second
    # game and so forth.
    #
    # Invocation examples:
    #
    #   ry game1
    #   ry --game2
    #   ry game1..20
    #
    # ======================================================================= #
    when /^-?-?game(\d{1,3}\.{0,2}\d{0,3})$/ # ← See: http://rubular.com/r/n2r8DwcTnU
      match = $1.to_s.dup
      i = return_this_game_component(match)
    # ======================================================================= #
    # === Compile a specific kde component
    #
    # This entry point allows us to use numbers in order to compile a
    # particular KDE component. So for example, "kde1" will be a
    # shortcut for "extracmakemodules".
    #
    # Invocation examples:
    #
    #   ry kde1
    #   ry kde2
    #   ry kde3
    #   ry --kde1
    #   ry --kde2
    #   ry --kde3
    #   ry --kde4
    #   ry --kde5
    #   ry --kde10
    #   ry kde11
    #   ry kde15
    #   ry kde1..100
    #   ry --kde1..kde250
    #
    # ======================================================================= #
    when /^-?-?kde(\d{1,3}\.{0,2}[a-z]{0,3}\d{0,3})/ # ← See: http://rubular.com/r/PzADEmm1Jc
      match = $1.to_s.dup
      i = return_this_kde_component(match)
    end
    i # Return it here.
  end

  # ========================================================================= #
  # === RBT.return_kde5_porting_aids
  #
  # This will return all KDE5 porting aids.
  #
  # Invoke this method like so:
  #
  #   ry --batch-compile=kde5_porting_aids
  #
  # ========================================================================= #
  def self.return_kde5_porting_aids
    result = return_chained_programs['kde5_porting_aids']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end; self.instance_eval { alias portingaids         return_kde5_porting_aids } # === RBT.portingaids
       self.instance_eval { alias return_porting_aids return_kde5_porting_aids } # === RBT.return_porting_aids

  # ========================================================================= #
  # === RBT.return_kde5_plasma
  #
  # This method will return all registered KDE5 plasma components, via
  # an Array. Whenever you need to obtain all KDE5-plasma components,
  # you should use only this method.
  #
  # In Januar 2020 there were 46 registered KDE5 plasma components.
  #
  # This can be queried via: RBT.return_kde5_plasma.size
  # ========================================================================= #
  def self.return_kde5_plasma
    result = return_chained_programs['kde5_plasma']
    if result.is_a? String
      result = result.split(',')
    end
    result.map!(&:strip)
    return result
  end; self.instance_eval { alias return_kde_plasma_components return_kde5_plasma } # === RBT.return_kde_plasma_components

  # ========================================================================= #
  # === RBT.return_kde5_foundation
  #
  # This method will return the Array of programs that constitute the
  # KDE5 foundation.
  # ========================================================================= #
  def self.return_kde5_foundation
    result = return_chained_programs['kde5_foundation']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end; self.instance_eval { alias kde5_foundation return_kde5_foundation } # === RBT.kde5_foundation

  # ========================================================================= #
  # === RBT.return_xorg_server_libraries
  # ========================================================================= #
  def self.return_xorg_server_libraries
    result = return_chained_programs[
      try_to_find_shortcut_for_a_chain_compiled_program('xorg_server_libraries')
    ]
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === RBT.return_this_lxqt_component
  #
  # The input to this method should typically be a number, such as
  # 1 (or its String-variant "1").
  #
  # Invocation example from the commandline:
  #
  #   ry --lxqt3
  #
  # ========================================================================= #
  def self.return_this_lxqt_component(i)
    result = nil
    big_array = return_lxqt_components
    big_array.flatten!
    if i.to_s.include? '..'
      splitted = i.split('..')
      first_number = splitted.first.to_i - 1
      last_number  = splitted.last.to_i - 1
      last_number  = (big_array.size - 1) if last_number > (big_array.size)
      result = big_array[first_number, last_number]
    else
      # ===================================================================== #
      # Prevent too high numbers next:
      # ===================================================================== #
      i = i.to_i # ← We need Integers past this point.
      i = (big_array.size - 1) if i > (big_array.size - 1)
      result = big_array[i - 1] # Find the proper result
    end
    result
  end

  # ========================================================================= #
  # === RBT.return_lxqt_components
  #
  # This method will conveniently return the Array holding all programs
  # that belong to the LX-QT pool.
  # ========================================================================= #
  def self.return_lxqt_components
    result = return_chained_programs['lxqt']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end; self.instance_eval { alias return_lxqt_desktop_components return_lxqt_components } # === RBT.return_lxqt_desktop_components

  # ========================================================================= #
  # === RBT.return_this_xorg_component
  #
  # The input to this method should typically be a number, such as
  # 1 (or its String-variant "1").
  #
  # Invocation example from the commandline:
  #
  #   ry --xorg1
  #
  # ========================================================================= #
  def self.return_this_xorg_component(i)
    result = nil
    big_array = return_xorg_components
    big_array.flatten!
    if i.to_s.include? '..'
      splitted = i.split('..')
      first_number = splitted.first.to_i - 1
      last_number  = splitted.last.to_i - 1
      last_number  = (big_array.size - 1) if last_number > (big_array.size)
      result = big_array[first_number, last_number]
    else
      # ===================================================================== #
      # Prevent too high numbers next:
      # ===================================================================== #
      i = i.to_i # ← We need Integers past this point.
      i = (big_array.size - 1) if i > (big_array.size - 1)
      result = big_array[i - 1] # Find the proper result
    end
    result
  end; self.instance_eval { alias try_to_compile_this_xorg_component         return_this_xorg_component } # === RBT.try_to_compile_this_xorg_component
       self.instance_eval { alias try_to_compile_this_xorg_desktop_component return_this_xorg_component } # === RBT.try_to_compile_this_xorg_desktop_component

  # ========================================================================= #
  # === RBT.return_xorg_libraries
  # ========================================================================= #
  def self.return_xorg_libraries
    result = return_chained_programs['xorg_libraries']
    result = result.split(',') if result.include? ','
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === RBT.return_xorg_data
  # ========================================================================= #
  def self.return_xorg_data
    result = return_chained_programs['xorg_data']
    result = result.split(',') if result.include? ','
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === RBT.return_xorg_apps
  # ========================================================================= #
  def self.return_xorg_apps
    result = return_chained_programs['xorg_apps']
    result = result.split(',') if result.include? ','
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === RBT.return_xorg_fonts
  # ========================================================================= #
  def self.return_xorg_fonts
    result = return_chained_programs['xorg_fonts']
    result = result.split(',') if result.include? ','
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === RBT.return_xorg_server_and_post_xorg_server
  # ========================================================================= #
  def self.return_xorg_server_and_post_xorg_server
    result = return_chained_programs['xorg_server_and_post_xorg_server']
    result = result.split(',') if result.include? ','
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === RBT.return_post_xorg_server_components
  # ========================================================================= #
  def self.return_post_xorg_server_components
    result = return_chained_programs['xorg_server_and_post_xorg_server']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end; self.instance_eval { alias report_post_xorg_server_components return_post_xorg_server_components } # === RBT.report_post_xorg_server_components

  # ========================================================================= #
  # === RBT.return_xorg_utils
  # ========================================================================= #
  def self.return_xorg_utils
    result = return_chained_programs['xorg_utils']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === RBT.return_this_plasma_component
  #
  # The input to this method should typically be a number, such as
  # 1 (or its String-variant "1").
  #
  # Invocation example from the commandline:
  #
  #   ry --plasma3
  #
  # ========================================================================= #
  def self.return_this_plasma_component(
      i = 1
    )
    result = nil
    big_array = return_kde5_plasma # === KDE5 Plasma
    big_array.flatten!
    if i.include? '..'
      splitted = i.split('..')
      first_number = splitted.first.to_i - 1
      last_number  = splitted.last.to_i - 1
      last_number  = (big_array.size - 1) if last_number > (big_array.size)
      result = big_array[first_number, last_number]
    else
      # ===================================================================== #
      # Prevent too high numbers next:
      # ===================================================================== #
      i = i.to_i # ← We need Integers past this point.
      i = (big_array.size - 1) if i > (big_array.size - 1)
      result = big_array[i - 1] # Find the proper result
    end
    result
  end

  # ========================================================================= #
  # === RBT.return_this_kde_component
  #
  # The input to this method should typically be a number, such as
  # 1 (or its String-variant "1").
  #
  # Invocation examples from the commandline:
  #
  #   ry --kde3
  #   ry --kde1..kde250
  #
  # ========================================================================= #
  def self.return_this_kde_component(
      i = 1
    )
    result = nil
    big_array  = return_kde5_foundation   # === KDE5 Foundation
    big_array << return_kde5_porting_aids # === KDE5 Porting Aids
    big_array << return_kde5_applications # === KDE5 Applications
    big_array << return_kde5_plasma       # === KDE5 Plasma
    big_array.flatten!
    if i.include? '..'
      # ===================================================================== #
      # First we chop away 'kde' if this string exists.
      # ===================================================================== #
      i.sub!(/kde/,'') if i.include? 'kde'
      splitted = i.split('..')
      first_number = splitted.first.to_i - 1
      last_number  = splitted.last.to_i - 1
      last_number  = (big_array.size - 1) if last_number > (big_array.size)
      result = big_array[first_number, last_number]
    else
      # ===================================================================== #
      # We need numbers past this point. We also have to prevent too
      # high numbers, which the following code will do for us.
      # ===================================================================== #
      i = i.to_i # ← We need Integers past this point.
      i = (big_array.size - 1) if i > (big_array.size - 1)
      result = big_array[i - 1] # Find the proper result
    end
    result
  end

  # ========================================================================= #
  # === return_gnome1
  # ========================================================================= #
  def self.return_gnome1
    result = return_chained_programs['gnome1']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === return_gnome2
  # ========================================================================= #
  def self.return_gnome2
    result = return_chained_programs['gnome2']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === return_gnome3
  # ========================================================================= #
  def self.return_gnome3
    result = return_chained_programs['gnome3']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === return_gnome4
  # ========================================================================= #
  def self.return_gnome4
    result = return_chained_programs['gnome4']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === RBT.return_gnome5
  # ========================================================================= #
  def self.return_gnome5
    result = return_chained_programs['gnome5']
    result = result.split(',') if result.is_a? String
    result.map!(&:strip)
    return result
  end

  # ========================================================================= #
  # === RBT.return_kde5_portingaids
  # ========================================================================= #
  def self.return_kde5_portingaids(
      i = 1
    )
    result = nil
    big_array = return_porting_aids
    big_array.flatten!
    if i.include? '..'
      splitted = i.split('..')
      first_number = splitted.first.to_i - 1
      last_number  = splitted.last.to_i - 1
      last_number  = (big_array.size - 1) if last_number > (big_array.size)
      result = big_array[first_number, last_number]
    else
      # ===================================================================== #
      # Prevent too high numbers next:
      # ===================================================================== #
      i = i.to_i # ← We need Integers past this point.
      i = (big_array.size - 1) if i > (big_array.size - 1)
      result = big_array[i - 1] # Find the proper result
    end
    result
  end; self.instance_eval { alias try_to_compile_this_portingaids_component return_kde5_portingaids } # === RBT.try_to_compile_this_portingaids_component
       self.instance_eval { alias return_this_portingaids_component         return_kde5_portingaids } # === RBT.return_this_portingaids_component

  # ========================================================================= #
  # === RBT.return_this_gnome_component
  #
  # The input to this method should typically be a number, such as
  # 1 (or its String-variant "1").
  #
  # Invocation example from the commandline:
  #
  #   ry --gnome1
  #
  # ========================================================================= #
  def self.return_this_gnome_component(
      i = 1
    )
    result = nil
    big_array = return_gnome_components
    big_array.flatten!
    if i.include? '..'
      splitted = i.split('..')
      first_number = splitted.first.to_i - 1
      last_number  = splitted.last.to_i - 1
      last_number  = (big_array.size - 1) if last_number > (big_array.size)
      result = big_array[first_number, last_number]
    else
      # ======================================================================= #
      # Prevent too high numbers next:
      # ======================================================================= #
      i = i.to_i # ← We need Integers past this point.
      i = (big_array.size - 1) if i > (big_array.size - 1)
      result = big_array[i - 1] # Find the proper result
    end
    result
  end; self.instance_eval { alias try_to_compile_this_gnome           return_this_gnome_component } # === RBT.try_to_compile_this_gnome
       self.instance_eval { alias try_to_compile_this_gnome_component return_this_gnome_component } # === RBT.try_to_compile_this_gnome_component

  # ========================================================================= #
  # === return_gnome_components
  #
  # This method will conveniently return the Array holding all programs
  # that belong to the Gnome stack.
  # ========================================================================= #
  def self.return_gnome_components
    array  = return_gnome1
    array << return_gnome2
    array << return_gnome3
    array << return_gnome4
    array << return_gnome5
    array.flatten!
    return array
  end; self.instance_eval { alias return_gnome return_gnome_components } # === RBT.return_gnome

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  e RBT.return_kde5_applications.size  
end