#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Autosymlink
#
# This class will do only two things, more or less:
#
#   1) symlink from lib64/ to lib/ in the specified target directory
#   2) run "rnsymc", that is, symlinking this again
#
# Usage example:
#
#   RBT::Autosymlink.new
#
# =========================================================================== #
# require 'rbt/utility_scripts/autosymlink.rb'
# RBT::Autosymlink.new(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class Autosymlink < Base # === RBT::Autosymlink

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === determine_target_directory
  # ========================================================================= #
  def determine_target_directory(i = return_pwd)
    @target_directory = i
  end

  # ========================================================================= #
  # === rnsymc
  # ========================================================================= #
  def rnsymc
    require 'rbt/utility_scripts/symlink_from_to_current/symlink_from_to_current.rb'
    RBT::SymlinkFromToCurrent.new
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    determine_target_directory
    one = @target_directory+'lib64'
    two = "#{@target_directory}lib"
    if File.directory?(one) and
      !File.exist?(two)
      opne "Symlinking from #{steelblue(one)} to #{olive(two)} next:"
      cd @target_directory
      symlink(
        from: 'lib64',
        to:   'lib'
      )
      rnsymc
    end
  end

  # ========================================================================= #
  # === RBT::Autosymlink[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Autosymlink.new(ARGV)
end # autosymlink