#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CreatePkgconfigFileForLua
#
# Usage example:
#
#   RBT::CreatePkgconfigFileForLua.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/create_pkgconfig_file_for_lua.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class CreatePkgconfigFileForLua < Base # === RBT::CreatePkgconfigFileForLua

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @use_this_prefix
    # ======================================================================= #
    @use_this_program_version = '5.3.5'
    # ======================================================================= #
    # === @use_this_prefix
    # ======================================================================= #
    @use_this_prefix = "#{programs_dir?}Lua/#{@use_this_program_version}"
    # ======================================================================= #
    # === @store_in_this_directory
    # ======================================================================= #
    @store_in_this_directory = "#{@use_this_prefix}/"
  end

  # ========================================================================= #
  # === create_the_lua_pkgconfig_file
  # ========================================================================= #
  def create_the_lua_pkgconfig_file
    @dataset = action(:sanitize_cookbook, :lua)
    into = @store_in_this_directory+'lib/pkgconfig/lua.pc'
    what = "V=5.3
R=#{@use_this_program_version}

prefix=#{@use_this_prefix}
INSTALL_BIN=${prefix}/bin
INSTALL_INC=${prefix}/include
INSTALL_LIB=${prefix}/lib
INSTALL_MAN=${prefix}/share/man/man1
INSTALL_LMOD=${prefix}/share/lua/${V}
INSTALL_CMOD=${prefix}/lib/lua/${V}
exec_prefix=${prefix}
libdir=${exec_prefix}/lib
includedir=${prefix}/include

Name: Lua
Description: #{@dataset.short_description?}
Version: ${R}
Requires:
Libs: -L${libdir} -llua -lm -ldl
Cflags: -I${includedir}
"
    write_what_into(what, into)
    if File.exist? into
      opne "#{rev}The file #{sfile(into)} #{rev}was generated."
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    create_the_lua_pkgconfig_file
  end

  # ========================================================================= #
  # === RBT::CreatePkgconfigFileForLua[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::CreatePkgconfigFileForLua.new(ARGV)
end # createpkgconfigfileforlua