#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ExtraInformation
#
# This class will show the extra_information entry from a given cookbook,
# such as "ruby.yml".
#
# Usage example:
#
#   RBT::ExtraInformation.new
#
# =========================================================================== #
# require 'rbt/utility_scripts/extra_information.rb'
# RBT::ExtraInformation.show
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class ExtraInformation < Base # === RBT::ExtraInformation

  require 'rbt/requires/require_the_cookbook_class.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    # ======================================================================= #
    # We will either instantiate a new cookbook, or we will use the dataset
    # from the expanded cookbook file.
    # ======================================================================= #
    _ = first_argument?
    if expanded_cookbook_file_exists_for?(_)
      # ===================================================================== #
      # This variant is faster, so it will be checked first.
      # ===================================================================== #
      file = path_to_this_expanded_cookbooks_dataset(_)
      use_this_key = 'extra_information'.to_sym
      dataset = YAML.load_file(file)[use_this_key]
    else
      dataset = action(:SanitizeCookbook, _) { :fast }.extra_information?
    end
    e dataset
  end

  # ========================================================================= #
  # === RBT::ExtraInformation[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end; self.instance_eval { alias show new } # === RBT::ExtraInformation.show

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::ExtraInformation.show(ARGV)
end # extrainformation ruby
    # einf binutils