#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::FeedbackBinariesOf
#
# This class will yield to us all binaries of a given program.
# =========================================================================== #
# require 'cookbooks/utility_scripts/feedback_binaries_of.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class FeedbackBinariesOf < Base # === RBT::FeedbackBinariesOf

  require 'rbt/requires/require_the_cookbook_class.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = 'ruby',
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i)
    i = i.join(' ').strip if i.is_a? Array
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end; alias search_term? input? # === search_term?

  # ========================================================================= #
  # === obtain_dataset
  # ========================================================================= #
  def obtain_dataset
    @dataset = action(:SanitizeCookbook, input?) { :fast }
  end

  # ========================================================================= #
  # === binaries?
  # ========================================================================= #
  def binaries?
    @binaries
  end

  # ========================================================================= #
  # === report_binaries
  # ========================================================================= #
  def report_binaries
    if binaries?.empty?
      opne 'We do not know whether `'+sfancy(search_term?)+
           '` will install any binaries.'
    else
      @binaries.each {|entry|
        e "  #{rev}#{royalblue('-')} #{steelblue(entry)}"
      }
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    obtain_dataset
    @binaries = @dataset.binaries?
    report_binaries
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::FeedbackBinariesOf.new(ARGV)
end # binariesof? ruby