#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::PurgeHeadersOfThisProgram
#
# Use this class if you want to purge header files (.h files) from the
# /usr/include/ directory.
#
# Be careful - and be sure to really want to use this class, before using it.
# Ideally also have some backups available.
#
# Usage example:
#
#   RBT::PurgeHeadersOfThisProgram.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/purge_headers_of_this_program.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class PurgeHeadersOfThisProgram < Base # === RBT::PurgeHeadersOfThisProgram

  require 'rbt/requires/require_the_cookbook_class.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_commandline_arguments(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @target_base_directory
    # ======================================================================= #
    @target_base_directory = '/usr/include/'
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    @cookbook = action(:SanitizeCookbook, first_argument?) { :fast }
    all_headers = @cookbook.headers? # <- Obtain all registered headers.
    opne 'This program has the following headers registered:'
    e
    all_headers.each {|entry| e "  - #{entry}" }
    e
    unless all_headers.empty?
      all_headers.each {|file|
        target = "#{@target_base_directory}#{File.basename(file)}"
        if File.exist?(target) and File.file?(target)
          opnn; remove_file(target, :be_verbose)
        end
      }
    end
  end

  # ========================================================================= #
  # === RBT::PurgeHeadersOfThisProgram[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::PurgeHeadersOfThisProgram.new(ARGV)
end # purgeheaders
    # purgeheaders yaml