#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::PurgeIncorrectYamlDirectoriesInTheProgramsHierarchy
#
# This class can be used to purge all incorrectly set yaml/ directories
# in the /home/Programs/ hierarchy (or wherever else the user has defined
# the $PROGRAMS directory to reside at).
#
# It is not a very important class - it was mostly necessary so that we
# can remove incorrectly stored files into the yaml-"database".
#
# Usage example:
#
#   RBT::PurgeIncorrectYamlDirectoriesInTheProgramsHierarchy.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/purge_incorrect_yaml_directories_in_the_programs_hierarchy.rb'
# RBT::PurgeIncorrectYamlDirectoriesInTheProgramsHierarchy.new(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class PurgeIncorrectYamlDirectoriesInTheProgramsHierarchy < Base # === RBT::PurgeIncorrectYamlDirectoriesInTheProgramsHierarchy

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @array_purge_these_directories
    # ======================================================================= #
    @array_purge_these_directories = []
    # ======================================================================= #
    # === @use_this_as_the_prefix
    #
    # The following variable can be used to refer to the main
    # programs-directory which is used to denote where this class
    # will operate. By default it will be the programs-directory
    # at /home/Programs/, or wherever else the user keeps his/her
    # programs directory at. The variable can be overruled
    # if need be.
    # ======================================================================= #
    @use_this_as_the_prefix = programs_dir?
  end

  # ========================================================================= #
  # === set_use_this_as_the_prefix
  # ========================================================================= #
  def set_use_this_as_the_prefix(i)
    i = i.first if i.is_a? Array
    i = i.dup if i.frozen?
    i << '/' unless i.end_with? '/'
    @use_this_as_the_prefix = i
  end; alias set_use_this_as_main_directory set_use_this_as_the_prefix # === set_use_this_as_main_directory

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      if File.directory?(i)
        set_use_this_as_main_directory(i)
      end
    end
  end

  # ========================================================================= #
  # === purge_these_directories
  # ========================================================================= #
  def purge_these_directories(
      i = @array_purge_these_directories
    )
    if i.empty?
      opne 'Everything is fine - no faulty yaml/ '\
           'directory could be found.'
    else
      opne "Found #{steelblue(i.size.to_s)} faulty directories."
      opne 'These will be removed next.'
      # ===================================================================== #
      # Iterate over the target directories next.
      # ===================================================================== #
      i.each {|this_directory_has_to_go|
        if File.directory?(this_directory_has_to_go) and
           this_directory_has_to_go.delete('/').end_with?('yaml')
          opne steelblue('Next removing the directory `')+
               sdir(this_directory_has_to_go)+
               steelblue('`.')
          remove_directory(this_directory_has_to_go)
        end
      }
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    # ======================================================================= #
    # Grab all relevant entries first. An incorrect entry may look like
    # this:
    #
    #   "/home/Programs/Gegl/yaml/"
    #
    # ======================================================================= #
    programs_dir = @use_this_as_the_prefix
    @array_purge_these_directories = Dir["#{programs_dir}**/yaml/"].select {|entry|
      entry.sub(/^#{programs_dir}/, '').count('/') == 2
    }
    if @array_purge_these_directories.empty?
      opne 'No incorrect yaml-directory was found. '+
           steelblue('\\o/')
    else
      purge_these_directories
    end
  end

  # ========================================================================= #
  # === RBT::PurgeIncorrectYamlDirectoriesInTheProgramsHierarchy[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::PurgeIncorrectYamlDirectoriesInTheProgramsHierarchy.new(ARGV)
end # purge_incorrect_yaml_directories_in_the_programs_hierarchy
    # purgeincorrectyamldirectoriesintheprogramshierarchy /Mount/USB1/Programs_64bit_glibc_2.23