#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ShowDependenciesOfThisLibrary
# =========================================================================== #
# require 'rbt/utility_scripts/show_dependencies_of_this_library.rb'
# RBT.show_dependencies_of_this_library(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class ShowDependenciesOfThisLibrary < RBT::Base # === RBT::Cookbooks::ShowDependenciesOfThisLibrary

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_commandline_arguments(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === report                                                   (report tag)
  # ========================================================================= #
  def report(
      i,
      search_term = first_argument?
    )
    search_term = search_term.to_s
    if i.empty?
      opnerev 'No library was found to depend on '+
              lightblue(search_term)+"#{rev}."
    else
      opnerev 'The following libraries have a dependency on '+
               lightblue(search_term)+rev+':'
      e
      i.each_with_index {|entry, index| index += 1
        e "  #{royalblue(index)}) #{steelblue(entry)}"
      }
      e
    end
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === sdoftl --show
      # ===================================================================== #
      when /-?-?show/
        e royalblue(
            File.read(
              action(:ldd_file?)
            )
          )
        exit
      end
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    menu
    dataset = action(:LddDataset)
    search_term = first?
    # ======================================================================= #
    # Need to find the selection next:
    # ======================================================================= #
    new_dataset = dataset.select {|key, value|
      value.include?(search_term)
    }
    report(new_dataset.keys, search_term)
  end

end

# =========================================================================== #
# === RBT.show_dependencies_of_this_library
# =========================================================================== #
def self.show_dependencies_of_this_library(i = ARGV)
  RBT::ShowDependenciesOfThisLibrary.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::ShowDependenciesOfThisLibrary.new(ARGV)
end # sdoftl /usr/lib/libiconv.so.2