#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ShowDescriptionAndExtraInformation
#
# This class will try to show the description of a program, and, if it
# exists, the extra information of said program as well, via the
# commandline.
# =========================================================================== #
# require 'rbt/utility_scripts/show_description_and_extra_information.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class ShowDescriptionAndExtraInformation < RBT::Base # === RBT::ShowDescriptionAndExtraInformation

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @pad_with_two_spaces_to_the_left
    #
    # If the following variable is true then we will use two ' ' to pad
    # the line.
    # ======================================================================= #
    @pad_with_two_spaces_to_the_left = true
  end

  # ========================================================================= #
  # === output_this_coloured_line
  # ========================================================================= #
  def output_this_coloured_line(line)
    result = ''.dup
    result << '  ' if @pad_with_two_spaces_to_the_left
    # ======================================================================= #
    # The right-unicode arrow will always be colourized.
    # ======================================================================= #
    if line.include?('→') 
      result << lightgreen(line)
    elsif line.start_with? '  '
      result << olive(line)
    else
      result << olivedrab(line)
    end
    e result
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    this_program = first_argument?
    if is_this_program_included?(this_program)
      dataset = action(:SanitizeCookbook, this_program) { :fast }
      # ===================================================================== #
      # We need the raw description next, as that one preserves the
      # newlines in the .yml file.
      # ===================================================================== #
      description       = dataset.raw_description?
      extra_information = dataset.extra_information?
      if description and !description.empty?
        e steelblue('Description:')
        e
        description.split(N).each {|line|
          output_this_coloured_line(line)
        }
        e
      end
      if extra_information and !extra_information.empty?
        e steelblue('Extra information:')
        e
        extra_information.split(N).each {|line|
          output_this_coloured_line(line)
        }
        e
      end
    else
      opne "#{rev}The program #{steelblue(this_program)} #{rev}is not included."
    end
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::ShowDescriptionAndExtraInformation.new(ARGV)
end # showdescriptionandextrainformation mesa