#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ShowManualSteps
#
# This small class will simply show the manual steps of a given program.
#
# In order for this to work, there has to be a manual_steps: entry in
# the corresponding .yml file.
#
# Usage example:
#
#   RBT::ShowManualSteps.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/show_manual_steps.rb'
# RBT::ShowManualSteps.new(ARGV)
# RBT.show_manual_steps
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class ShowManualSteps < Base # === RBT::ShowManualSteps

  require 'rbt/requires/require_the_cookbook_class.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    _ = first_argument?
    cookbook = RBT.cookbook_for(_)
    manual_steps = cookbook.manual_steps?
    if manual_steps and !manual_steps.empty?
      report_these_manual_steps(manual_steps, _)
    else
      opne "#{rev}No manual steps are known for #{sfancy(_)}#{rev}."
    end
  end

  # ========================================================================= #
  # === report_these_manual_steps (report tag)
  # ========================================================================= #
  def report_these_manual_steps(i, this_program)
    opne "#{rev}The following manual steps are known for the "\
         "program #{steelblue(this_program)}#{rev}:"
    if i.is_a? String
      i = i.split(N)
    end
    compacted_array = [i].flatten.compact
    compacted_array.shift if compacted_array.first.strip.empty? # Get rid of leading newlines here.
    padded = compacted_array.map {|entry|
      entry = entry.dup
      entry.prepend('  ')
      entry
    }
    e
    padded.each {|line|
      e mediumseagreen(line)
    }
    e
  end; alias report report_these_manual_steps # === report

  # ========================================================================= #
  # === RBT::ShowManualSteps.of
  # ========================================================================= #
  def self.of(i = ARGV)
    new(i)
  end; self.instance_eval { alias [] of } # === ShowManualSteps[]

end

# =========================================================================== #
# === RBT.show_manual_steps
#
# Usage example:
#
#   RBT.show_manual_steps(:kde5_plasma)
#
# =========================================================================== #
def self.show_manual_steps(of = ARGV)
  [of].flatten.compact.each {|this_program|
    RBT::ShowManualSteps.new(this_program)
  }
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::ShowManualSteps.new(ARGV)
end # showmanualsteps glibc