#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::SymlinkIntoTheUsrBinHierarchy
#
# This class can be used to symlink the designated input-files into the
# /usr/bin/ hierarchy. Invoke this class only if you are sure that
# you really want/need this functionality.
#
# You can also pass the path towards a directory, in which case this
# class will grab all entries of said directory.
#
# Usage example:
#
#   RBT::SymlinkIntoTheUsrBinHierarchy.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/symlink_into_the_usr_bin_hierarchy.rb'
# RBT::SymlinkIntoTheUsrBinHierarchy.new(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class SymlinkIntoTheUsrBinHierarchy < Base # === RBT::SymlinkIntoTheUsrBinHierarchy

  # ========================================================================= #
  # === USR_BIN_DIRECTORY
  # ========================================================================= #
  USR_BIN_DIRECTORY = '/usr/bin/'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      symlink_these_files = nil,
      run_already         = true,
      &block
    )
    reset
    set_symlink_these_files(
      symlink_these_files
    )
    # ======================================================================= #
    # === Handle blocks given to this class next
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :force
      # ===================================================================== #
      when :force
        @internal_hash[:remove_the_target_file_at_the_usr_bin_hierarchy] = true
      else
        if yielded.is_a?(Hash)
          # ================================================================= #
          # === :force_symlink
          # ================================================================= #
          if yielded.has_key? :force_symlink
            @internal_hash[:remove_the_target_file_at_the_usr_bin_hierarchy] = yielded.delete(:force_symlink)
          end
        end
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ========================================================================= #
    # === :remove_the_target_file_at_the_usr_bin_hierarchy
    # ========================================================================= #
    @internal_hash[:remove_the_target_file_at_the_usr_bin_hierarchy] = false
  end

  # ========================================================================= #
  # === remove_the_target_file_at_the_usr_bin_hierarchy?
  # ========================================================================= #
  def remove_the_target_file_at_the_usr_bin_hierarchy?
    @internal_hash[:remove_the_target_file_at_the_usr_bin_hierarchy]
  end

  # ========================================================================= #
  # === set_symlink_these_files
  # ========================================================================= #
  def set_symlink_these_files(i = '')
    if i
      i = [i].flatten.compact.map {|entry| # Always have an Array past this point.
        if File.directory? entry # If it is a directory we'll grab all entries from it.
          target = ("#{entry}/*").squeeze '/'
          entry = Dir[target] # ← Obtain the files from that directory.
        end
        entry
      }
      i.flatten!
      i.compact!
    end
    @internal_hash[:symlink_these_files] = i
  end

  # ========================================================================= #
  # === symlink_these_files?
  # ========================================================================= #
  def symlink_these_files?
    @internal_hash[:symlink_these_files]
  end

  # ========================================================================= #
  # === do_work_on_these_files
  # ========================================================================= #
  def do_work_on_these_files(
      i = symlink_these_files?
    )
    remove_the_target_file_at_the_usr_bin_hierarchy = remove_the_target_file_at_the_usr_bin_hierarchy?
    i.each {|this_file|
      # ===================================================================== #
      # The variable this_file may contain a file such as
      # /Programs/Ruby/Current/lib/libruby.so.2.5.3.
      # ===================================================================== #
      target = "#{USR_BIN_DIRECTORY}#{File.basename(this_file)}" # This may be a String such as "/usr/bin/htop"
      orev 'Next symlinking the file `'+sfile(this_file)+
           rev+'` into `'+sfile(target)+'`.'
      # ===================================================================== #
      # Next, handle the case when the target at `target` is a directory:
      # ===================================================================== #
      if File.directory? this_file
        if File.directory? target
          # ================================================================= #
          # In this case the target directory already exists.
          # ================================================================= #
          Dir["#{this_file}/*"].each {|target_in_that_directory|
            inner_target = target+'/'+File.basename(target_in_that_directory)
            orev 'Symlinking '+sfile(target_in_that_directory)+
                 rev+' into `'+sfancy(inner_target)+rev+'` next.'
            symlink(
              target_in_that_directory,
              inner_target
            )
          }
          return
        end
        target = this_file
      else
        if File.file?(target) and
           remove_the_target_file_at_the_usr_bin_hierarchy and
           File.file?(this_file) # This check here is necessary to find out that the file exists.
          delete_file(target)
        end
      end
      symlink(this_file, target)
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    do_work_on_these_files(symlink_these_files?)
  end

  # ========================================================================= #
  # === RBT::SymlinkIntoTheUsrBinHierarchy[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::SymlinkIntoTheUsrBinHierarchy.new(ARGV)
end # SymlinkIntoTheUsrBinHierarchy $MY_PROGRAMS/Ruby/Current/lib/
    # SymlinkIntoTheUsrBinHierarchy $MY_PROGRAMS/Wget/Current/bin/