#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::SymlinkIntoUsrLibDirectory
#
# This class can be used to symlink the designated input-files into the
# /usr/lib/ hierarchy. Invoke this class only if you are sure that
# you really want/need this functionality.
#
# You can also pass the path towards a directory, in which case this
# class # will grab all entries of said directory.
#
# Usage example:
#
#   RBT::SymlinkIntoUsrLibDirectory.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/symlink_into_usr_lib_directory.rb'
# RBT::SymlinkIntoUsrLibDirectory.new(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class SymlinkIntoUsrLibDirectory < Base # === RBT::SymlinkIntoUsrLibDirectory

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      symlink_these_files = nil,
      run_already         = true
    )
    reset
    set_symlink_these_files(
      symlink_these_files
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === set_symlink_these_files
  # ========================================================================= #
  def set_symlink_these_files(i = '')
    if i
      i = [i].flatten.compact.map {|entry|
        if File.directory? entry
          target = ("#{entry}/*").squeeze '/'
          entry = Dir[target] # ← Obtain the files from that directory.
        end
        entry
      }
      i.flatten!
    end
    @symlink_these_files = i
  end

  # ========================================================================= #
  # === symlink_these_files?
  # ========================================================================= #
  def symlink_these_files?
    @symlink_these_files
  end

  # ========================================================================= #
  # === do_symlink_the_following_files
  # ========================================================================= #
  def do_symlink_the_following_files(i = @symlink_these_files)
    i.each {|this_file|
      # ===================================================================== #
      # The variable this_file may contain a file such as
      # /Programs/Ruby/Current/lib/libruby.so.2.5.3.
      # ===================================================================== #
      target = "/usr/lib/#{File.basename(this_file)}"
      opne 'Next symlinking the file `'+sfile(this_file)+
           '` into `'+sfile(target)+'`.'
      if File.directory? this_file
        if File.directory? target
          # ================================================================= #
          # In this case the target directory already exists.
          # ================================================================= #
          Dir["#{this_file}/*"].each {|target_in_that_directory|
            inner_target = target+'/'+File.basename(target_in_that_directory)
            opne 'Symlinking '+sfile(target_in_that_directory)+
                 ' into `'+sfancy(inner_target)+'` next.'
            symlink(
              target_in_that_directory,
              inner_target
            )
          }
          return
        end
        target = this_file
      end
      symlink(this_file, target)
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    do_symlink_the_following_files(@symlink_these_files)
  end

  # ========================================================================= #
  # === RBT::SymlinkIntoUsrLibDirectory[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::SymlinkIntoUsrLibDirectory.new(ARGV)
end # symlinkintousrlibdirectory $MY_PROGRAMS/Ruby/Current/lib/