#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Uninstall
#
# The idea behind this class is to uninstall a given program. This currently
# does not work very well, though.
#
# Usage example:
#
#   RBT::Uninstall.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/uninstall.rb'
# RBT::Uninstall.new(ARGV)
# RBT.uninstall(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class Uninstall < Base # === RBT::Uninstall

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    set_commandline_arguments(commandline_arguments)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === uninstall_this_directory
  # ========================================================================= #
  def uninstall_this_directory(i)
    i = rds(i).dup
    unless i == '/'
      e "#{rev}Removing the directory #{sdir(i)} #{rev}next."
      remove_directory(i)
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    _ = first_argument?
    if _
      # ===================================================================== #
      # Handle directories here, if we are in the programs_dir? hierarchy.
      # ===================================================================== #
      if File.directory?(_) and
         File.absolute_path(_).include?(programs_dir?) and
         !(return_pwd == '/')
        e "#{rev}Uninstalling the program stored at #{sdir(_)} #{rev}next."
        uninstall_this_directory(_)
      elsif File.file?(_)
        # So it is a file. Let's see whether it is registered.
        dataset_from_libraries = YAML.load_file(yaml_dir?+'registered/libraries.yml')
        if dataset_from_libraries.has_key? _
          points_towards = dataset_from_libraries[_]
          e "#{rev}This is a registered library. It points towards "+
            steelblue(points_towards)+'.'
          e 'It will be uninstalled next.'
          e rev+'All libraries that belong to '+steelblue(points_towards)+
            rev+' will be collected next.'
          selection = dataset_from_libraries.select {|key, value|
            value == points_towards
          }
          pp selection
e 'TODO: CONTINUE HERE'
        end
      end
    else
      e "#{rev}Please supply the program that you wish to uninstall."
    end
  end

  # ========================================================================= #
  # === RBT::Uninstall[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end

# =========================================================================== #
# === RBT.uninstall
# =========================================================================== #
def self.uninstall(i = ARGV)
  RBT::Uninstall.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Uninstall.new(ARGV)
end # Uninstall python